import json
from xml.etree import cElementTree as eTree

import sysactions
from mw_helper import show_confirm_dialog

from .. import logger
from .. import mb_context

from bustoken import TK_REMOTE_ORDER_CONFIRM_DELIVERY_PAYMENT
from msgbus import TK_SYS_ACK


@sysactions.action
def confirm_delivery_payment(pos_id, order_id, external_id):
    from posactions import ask_for_delivery_payment

    order_taker = None
    try:
        model = sysactions.get_model(pos_id)
        order_taker = sysactions.get_posot(model)
        message = "$CONFIRM_DELIVERY_PAYMENT"
        order_pict = order_taker.orderPicture(orderid=order_id)
        order = eTree.XML(order_pict).find("Order")
        order_state = order.attrib['state']
        tender_prepaid = _get_tender_prepaid(order_taker, order_id)

        if (tender_prepaid.lower() != 'true' and order_state != 'VOIDED') and ask_for_delivery_payment:
            if not _set_order_tenders(pos_id, order_id, message, order, order_taker):
                return False
        else:
            message = "$CLOSE_DELIVERY_ORDER_QUESTION"
            confirmation = sysactions.show_messagebox(pos_id, message, timeout=720000, buttons="$OK|$CANCEL")
            if confirmation == 1:
                return False

        xml = "|".join([order_id, external_id])
        mb_context.MB_EasyEvtSend("ConfirmDeliveryPayment", type="", xml=xml, sourceid=int(pos_id))

        confirm_delivery_payment_server(order_id, pos_id)

    except Exception as ex:
        sysactions.show_messagebox(pos_id, "$ERROR_ON_REMOTE_ORDER_DELIVERY_CONFIRMATION|{}".format(ex))
        logger.error(ex)
        return False
    finally:
        order_taker.blkopnotify = False

    return True


def _set_order_tenders(pos_id, order_id, message, order, order_taker):
    tenders = show_confirm_dialog(pos_id, title=message)
    tenders = None if tenders is None else json.loads(tenders)

    if tenders is None:
        return False
    success, new_tenders = _get_new_tenders(tenders, pos_id, order)
    if success is False:
        return False

    order_taker.blkopnotify = True
    order_pos_id = int(order.attrib["originatorId"][3:])
    model_delivery = sysactions.get_model(order_pos_id)
    order_taker_delivery = sysactions.get_posot(model_delivery)
    try:
        returned_tenders = order_taker_delivery.resetOrderTenders(order_id, new_tenders)
    except Exception as _:
        sysactions.show_messagebox(pos_id, '$DELIVERY_CONFIRM_ERROR_NOT_PAID', timeout=720000)
        return False
    logger.debug(returned_tenders)
    order_taker.blkopnotify = False
    if returned_tenders:
        total_amount = float(returned_tenders['totalAmount'])
        new_total_tender = float(returned_tenders['totalTender'])
        if new_total_tender > total_amount:
            sysactions.show_messagebox(pos_id, returned_tenders['change'], title='$NEW_CHANGE', timeout=720000)
    else:
        return False

    return True


def _get_tender_prepaid(order_taker, order_id):
    tender_prepaid = order_taker.getOrderCustomProperties(key="PREPAID", orderid=order_id)
    tender_prepaid = eTree.XML(tender_prepaid).find("OrderProperty")
    tender_prepaid = 'true' if tender_prepaid is None else tender_prepaid.get('value')
    return tender_prepaid


def _get_new_tenders(tenders, pos_id, order):
    new_tenders = []
    for value in tenders:
        amount = float(value["amount"])
        tender_id = int(value["tenderId"])
        if amount != 0:
            tender = {}
            value = "{:.2f}".format(round(amount, 2))
            tender['tenderid'] = tender_id
            tender['amount'] = value
            tender['tip'] = '0.00'
            tender['tenderdetail'] = 'confirmDelivery'
            new_tenders.append(tender)
    if not new_tenders:
        sysactions.show_messagebox(pos_id, "$NO_PAYMENT_DETECTED")
        return False, new_tenders
    for original_tender in order.findall('TenderHistory/Tender'):
        if original_tender.attrib['tenderType'] not in ("0", "201"):
            tender = {}
            value = "{:.2f}".format(round(float(original_tender.attrib['tenderAmount']), 2))
            tender['tenderid'] = int(original_tender.attrib['tenderType'])
            tender['amount'] = value
            tender['tip'] = '0.00'
            tender['tenderdetail'] = 'confirmDelivery'
            new_tenders.append(tender)

    return True, new_tenders


def confirm_delivery_payment_server(order_id, pos_id):
    token = TK_REMOTE_ORDER_CONFIRM_DELIVERY_PAYMENT
    msg = mb_context.MB_EasySendMessage("RemoteOrder", token, data=order_id, timeout=10000000)
    success = msg.token == TK_SYS_ACK
    if success:
        sysactions.show_messagebox(pos_id, "$SUCCESS_CONFIRM_REMOTE_ORDER_DELIVERY", timeout=720000, buttons="$OK")
    else:
        sysactions.show_messagebox(pos_id, "$ERROR_ON_CONFIRM_REMOTE_ORDER_DELIVERY", timeout=720000, buttons="$OK")



from typing import Dict, Tuple, Optional


class FiscalParameterNotFound(Exception):
    def __init__(self, part_code, parameter_name):
        # type: (str, unicode) -> None

        super(FiscalParameterNotFound, self).__init__("FiscalParameterNotFound")
        self.part_code = str(part_code)
        self.parameter_name = parameter_name

    def __str__(self):
        return "FiscalParameterNotFound: partCode: {0} - parameterName: {1}".format(self.part_code, self.parameter_name)


class FiscalParameterController(object):
    def __init__(self, fiscal_parameters):
        self.fiscal_parameters = fiscal_parameters  # type: Dict[Tuple[str, unicode],unicode]

    def get_optional_parameter(
            self,
            part_code,      # type: str
            parameter_name  # type: unicode
    ):
        # type: (...) -> Optional[unicode]

        key = (str(part_code), parameter_name)
        if key not in self.fiscal_parameters:
            return None
        else:
            return self.fiscal_parameters[key]

    def get_parameter(
            self,
            part_code,      # type: str
            parameter_name  # type: unicode
    ):
        # type: (...) -> unicode

        key = (str(part_code), parameter_name)
        if key not in self.fiscal_parameters:
            raise FiscalParameterNotFound(part_code, parameter_name)
        else:
            return self.fiscal_parameters[key]

    def has_parameter(
            self,
            part_code,      # type: str
            parameter_name  # type: unicode
    ):
        # type: (...) -> bool

        key = (str(part_code), parameter_name)
        return key in self.fiscal_parameters

    def get_all_products_csts(self):
        # type: () -> unicode

        return str({(x, y): self.fiscal_parameters.get((x, y)) for x, y in self.fiscal_parameters if "CST" in y})

    def get_all_products_base_reductions(self):
        # type: () -> unicode

        return str({x: self.fiscal_parameters.get((x, "BASE_REDUCTION")) for x, y in self.fiscal_parameters})



from abc import ABCMeta, abstractmethod
from datetime import datetime

from production.model import ProductionOrder
from typing import List


class ProductionRepository(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def save_order(self, order):
        # type: (ProductionOrder) -> None

        raise NotImplementedError()

    def get_all_orders(self):
        # type: () -> List[ProductionOrder]

        raise NotImplementedError()

    def get_orders_to_purge(self, limit_date):
        # type: (datetime) -> List[ProductionOrder]

        raise NotImplementedError()

    def purge_orders(self, orders):
        # type: (List[ProductionOrder]) -> None

        raise NotImplementedError()

    def stop(self):
        # type: () -> None

        raise NotImplementedError()



from logging import Logger
from mw_helper import convert_to_dict, ensure_list
from production.box import ProductionBox
from production.box._OrderChangerProductionBox import OrderChangerProductionBox
from production.model import ProductionOrder
from typing import Optional, List, Union


class OrderPropertiesFilterBox(OrderChangerProductionBox):
    def __init__(
            self,
            name,                       # type: str
            sons,                       # type: Union[List[ProductionBox], ProductionBox]
            order_property,             # type: str
            allowed_values=None,        # type: Optional[List[str]]
            excluded_values=None,       # type: Optional[List[str]]
            remove_order=True,          # type: bool
            logger=None                 # type: Optional[Logger]
    ):
        # type: (...) -> None

        super(OrderPropertiesFilterBox, self).__init__(name, sons, logger)
        self.order_property = order_property
        self.allowed_values = convert_to_dict(ensure_list(allowed_values))
        self.excluded_values = convert_to_dict(ensure_list(excluded_values))
        self.remove_order = remove_order

    def change_order(
            self,
            order   # type: ProductionOrder
    ):
        # type: (...) -> Optional[ProductionOrder]

        if len(self.excluded_values) > 0 and str(getattr(order, self.order_property)) in self.excluded_values:
            if self.remove_order:
                return None
            else:
                order.items = []

        if len(self.allowed_values) > 0 and str(getattr(order, self.order_property)) not in self.allowed_values:
            if self.remove_order:
                return None
            else:
                order.items = []

        return order



import os

import pyscripts
from cfgtools import read
from helper import import_pydevd, config_logger
from logging import getLogger
from msgbus import MBEasyContext
from mbcontextmessagehandler import MbContextMessageBus, MbContextMessageHandler

from externals.messagehandlerbuilder import ComponentMessageHandlerBuilder
from externals.messages import ListenedEvents


LOADER_CFG = os.environ["LOADERCFG"]


def main():
    import_pydevd(LOADER_CFG, 9127, False)

    config = read(LOADER_CFG)

    goals_filename = config.find_value("DailyGoals.goalsCacheFileName")

    service_name = str(config.find_value("Identification.ComponentName"))
    required_services = "|".join(config.find_values(service_name + "." + "RequiredServices"))
    persistence_name = config.find_value(service_name + "." + "RequiredServices").encode("utf-8")

    mb_context = MBEasyContext(service_name)
    pyscripts.mbcontext = mb_context

    config_logger(os.environ["LOG_PATH"], service_name)
    logger = getLogger(__name__)

    message_bus = MbContextMessageBus(mb_context)
    message_handler_builder = ComponentMessageHandlerBuilder(mb_context, persistence_name, goals_filename)

    message_handler = MbContextMessageHandler(
        message_bus,
        service_name,
        service_name,
        required_services,
        message_handler_builder
    )

    logger.warning("Starting {} Component".format(service_name))

    message_handler.subscribe_reentrant_events([ListenedEvents.ORDER_MODIFIED.value, ListenedEvents.TABLE_CLOSED.value])
    message_handler.handle_events()


if __name__ == "__main__":
    main()



from ._Wait import wait
from ._User import get_user_data
from ._Range import get_range_discount_ids
from ._DictionaryOperations import DictionaryOperations
from ._ServicePeriod import get_service_period
from ._CompositionType import CompositionType



from nperipherals_core.model.device import CashDrawer


class CashDrawerOpenInteractor(object):
    def __init__(self, device):
        # type: (CashDrawer) -> None

        self.device = device

    def open_drawer(self):
        connected = False
        try:
            self.device.initialize()
            self.device.connect()
            connected = True

            self.device.open_drawer()
            return 0

        finally:
            if connected:
                self.device.disconnect()



r"""UUID objects (universally unique identifiers) according to RFC 4122.

This module provides immutable UUID objects (class UUID) and the functions
uuid1(), uuid3(), uuid4(), uuid5() for generating version 1, 3, 4, and 5
UUIDs as specified in RFC 4122.

If all you want is a unique ID, you should probably call uuid1() or uuid4().
Note that uuid1() may compromise privacy since it creates a UUID containing
the computer's network address.  uuid4() creates a random UUID.

Typical usage:

    >>> import uuid

    # make a UUID based on the host ID and current time
    >>> uuid.uuid1()
    UUID('a8098c1a-f86e-11da-bd1a-00112444be1e')

    # make a UUID using an MD5 hash of a namespace UUID and a name
    >>> uuid.uuid3(uuid.NAMESPACE_DNS, 'python.org')
    UUID('6fa459ea-ee8a-3ca4-894e-db77e160355e')

    # make a random UUID
    >>> uuid.uuid4()
    UUID('16fd2706-8baf-433b-82eb-8c7fada847da')

    # make a UUID using a SHA-1 hash of a namespace UUID and a name
    >>> uuid.uuid5(uuid.NAMESPACE_DNS, 'python.org')
    UUID('886313e1-3b8a-5372-9b90-0c9aee199e5d')

    # make a UUID from a string of hex digits (braces and hyphens ignored)
    >>> x = uuid.UUID('{00010203-0405-0607-0809-0a0b0c0d0e0f}')

    # convert a UUID to a string of hex digits in standard form
    >>> str(x)
    '00010203-0405-0607-0809-0a0b0c0d0e0f'

    # get the raw 16 bytes of the UUID
    >>> x.bytes
    '\x00\x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\x0c\r\x0e\x0f'

    # make a UUID from a 16-byte string
    >>> uuid.UUID(bytes=x.bytes)
    UUID('00010203-0405-0607-0809-0a0b0c0d0e0f')
"""

__author__ = 'Ka-Ping Yee <ping@zesty.ca>'
__date__ = '$Date: 2006/06/12 23:15:40 $'.split()[1].replace('/', '-')
__version__ = '$Revision: 1.30 $'.split()[1]

RESERVED_NCS, RFC_4122, RESERVED_MICROSOFT, RESERVED_FUTURE = [
    'reserved for NCS compatibility', 'specified in RFC 4122',
    'reserved for Microsoft compatibility', 'reserved for future definition']

class UUID(object):
    """Instances of the UUID class represent UUIDs as specified in RFC 4122.
    UUID objects are immutable, hashable, and usable as dictionary keys.
    Converting a UUID to a string with str() yields something in the form
    '12345678-1234-1234-1234-123456789abc'.  The UUID constructor accepts
    four possible forms: a similar string of hexadecimal digits, or a
    string of 16 raw bytes as an argument named 'bytes', or a tuple of
    six integer fields (with 32-bit, 16-bit, 16-bit, 8-bit, 8-bit, and
    48-bit values respectively) as an argument named 'fields', or a single
    128-bit integer as an argument named 'int'.

    UUIDs have these read-only attributes:

        bytes       the UUID as a 16-byte string

        fields      a tuple of the six integer fields of the UUID,
                    which are also available as six individual attributes
                    and two derived attributes:

            time_low                the first 32 bits of the UUID
            time_mid                the next 16 bits of the UUID
            time_hi_version         the next 16 bits of the UUID
            clock_seq_hi_variant    the next 8 bits of the UUID
            clock_seq_low           the next 8 bits of the UUID
            node                    the last 48 bits of the UUID

            time                    the 60-bit timestamp
            clock_seq               the 14-bit sequence number

        hex         the UUID as a 32-character hexadecimal string

        int         the UUID as a 128-bit integer

        urn         the UUID as a URN as specified in RFC 4122

        variant     the UUID variant (one of the constants RESERVED_NCS,
                    RFC_4122, RESERVED_MICROSOFT, or RESERVED_FUTURE)

        version     the UUID version number (1 through 5, meaningful only
                    when the variant is RFC_4122)
    """

    def __init__(self, hex=None, bytes=None, fields=None, int=None,
                       version=None):
        r"""Create a UUID from either a string of 32 hexadecimal digits,
        a string of 16 bytes as the 'bytes' argument, a tuple of six
        integers (32-bit time_low, 16-bit time_mid, 16-bit time_hi_version,
        8-bit clock_seq_hi_variant, 8-bit clock_seq_low, 48-bit node) as
        the 'fields' argument, or a single 128-bit integer as the 'int'
        argument.  When a string of hex digits is given, curly braces,
        hyphens, and a URN prefix are all optional.  For example, these
        expressions all yield the same UUID:

        UUID('{12345678-1234-5678-1234-567812345678}')
        UUID('12345678123456781234567812345678')
        UUID('urn:uuid:12345678-1234-5678-1234-567812345678')
        UUID(bytes='\x12\x34\x56\x78'*4)
        UUID(fields=(0x12345678, 0x1234, 0x5678, 0x12, 0x34, 0x567812345678))
        UUID(int=0x12345678123456781234567812345678)

        Exactly one of 'hex', 'bytes', 'fields', or 'int' must be given.
        The 'version' argument is optional; if given, the resulting UUID
        will have its variant and version number set according to RFC 4122,
        overriding bits in the given 'hex', 'bytes', 'fields', or 'int'.
        """

        if [hex, bytes, fields, int].count(None) != 3:
            raise TypeError('need just one of hex, bytes, fields, or int')
        if hex is not None:
            hex = hex.replace('urn:', '').replace('uuid:', '')
            hex = hex.strip('{}').replace('-', '')
            if len(hex) != 32:
                raise ValueError('badly formed hexadecimal UUID string')
            int = long(hex, 16)
        if bytes is not None:
            if len(bytes) != 16:
                raise ValueError('bytes is not a 16-char string')
            int = long(('%02x'*16) % tuple(map(ord, bytes)), 16)
        if fields is not None:
            if len(fields) != 6:
                raise ValueError('fields is not a 6-tuple')
            (time_low, time_mid, time_hi_version,
             clock_seq_hi_variant, clock_seq_low, node) = fields
            if not 0 <= time_low < 1<<32L:
                raise ValueError('field 1 out of range (need a 32-bit value)')
            if not 0 <= time_mid < 1<<16L:
                raise ValueError('field 2 out of range (need a 16-bit value)')
            if not 0 <= time_hi_version < 1<<16L:
                raise ValueError('field 3 out of range (need a 16-bit value)')
            if not 0 <= clock_seq_hi_variant < 1<<8L:
                raise ValueError('field 4 out of range (need an 8-bit value)')
            if not 0 <= clock_seq_low < 1<<8L:
                raise ValueError('field 5 out of range (need an 8-bit value)')
            if not 0 <= node < 1<<48L:
                raise ValueError('field 6 out of range (need a 48-bit value)')
            clock_seq = (clock_seq_hi_variant << 8L) | clock_seq_low
            int = ((time_low << 96L) | (time_mid << 80L) |
                   (time_hi_version << 64L) | (clock_seq << 48L) | node)
        if int is not None:
            if not 0 <= int < 1<<128L:
                raise ValueError('int is out of range (need a 128-bit value)')
        if version is not None:
            if not 1 <= version <= 5:
                raise ValueError('illegal version number')
            # Set the variant to RFC 4122.
            int &= ~(0xc000 << 48L)
            int |= 0x8000 << 48L
            # Set the version number.
            int &= ~(0xf000 << 64L)
            int |= version << 76L
        self.__dict__['int'] = int

    def __cmp__(self, other):
        if isinstance(other, UUID):
            return cmp(self.int, other.int)
        return NotImplemented

    def __hash__(self):
        return hash(self.int)

    def __int__(self):
        return self.int

    def __repr__(self):
        return 'UUID(%r)' % str(self)

    def __setattr__(self, name, value):
        raise TypeError('UUID objects are immutable')

    def __str__(self):
        hex = '%032x' % self.int
        return '%s-%s-%s-%s-%s' % (
            hex[:8], hex[8:12], hex[12:16], hex[16:20], hex[20:])

    def get_bytes(self):
        bytes = ''
        for shift in range(0, 128, 8):
            bytes = chr((self.int >> shift) & 0xff) + bytes
        return bytes

    bytes = property(get_bytes)

    def get_fields(self):
        return (self.time_low, self.time_mid, self.time_hi_version,
                self.clock_seq_hi_variant, self.clock_seq_low, self.node)

    fields = property(get_fields)

    def get_time_low(self):
        return self.int >> 96L

    time_low = property(get_time_low)

    def get_time_mid(self):
        return (self.int >> 80L) & 0xffff

    time_mid = property(get_time_mid)

    def get_time_hi_version(self):
        return (self.int >> 64L) & 0xffff

    time_hi_version = property(get_time_hi_version)

    def get_clock_seq_hi_variant(self):
        return (self.int >> 56L) & 0xff

    clock_seq_hi_variant = property(get_clock_seq_hi_variant)

    def get_clock_seq_low(self):
        return (self.int >> 48L) & 0xff

    clock_seq_low = property(get_clock_seq_low)

    def get_time(self):
        return (((self.time_hi_version & 0x0fffL) << 48L) |
                (self.time_mid << 32L) | self.time_low)

    time = property(get_time)

    def get_clock_seq(self):
        return (((self.clock_seq_hi_variant & 0x3fL) << 8L) |
                self.clock_seq_low)

    clock_seq = property(get_clock_seq)

    def get_node(self):
        return self.int & 0xffffffffffff

    node = property(get_node)

    def get_hex(self):
        return '%032x' % self.int

    hex = property(get_hex)

    def get_urn(self):
        return 'urn:uuid:' + str(self)

    urn = property(get_urn)

    def get_variant(self):
        if not self.int & (0x8000 << 48L):
            return RESERVED_NCS
        elif not self.int & (0x4000 << 48L):
            return RFC_4122
        elif not self.int & (0x2000 << 48L):
            return RESERVED_MICROSOFT
        else:
            return RESERVED_FUTURE

    variant = property(get_variant)

    def get_version(self):
        # The version bits are only meaningful for RFC 4122 UUIDs.
        if self.variant == RFC_4122:
            return int((self.int >> 76L) & 0xf)

    version = property(get_version)

def _ifconfig_getnode():
    """Get the hardware address on Unix by running ifconfig."""
    import os
    for dir in ['', '/sbin/', '/usr/sbin']:
        try:
            pipe = os.popen(os.path.join(dir, 'ifconfig'))
        except IOError:
            continue
        for line in pipe:
            words = line.lower().split()
            for i in range(len(words)):
                if words[i] in ['hwaddr', 'ether']:
                    return int(words[i + 1].replace(':', ''), 16)

def _ipconfig_getnode():
    """Get the hardware address on Windows by running ipconfig.exe."""
    import os, re
    dirs = ['', r'c:\windows\system32', r'c:\winnt\system32']
    try:
        import ctypes
        buffer = ctypes.create_string_buffer(300)
        ctypes.windll.kernel32.GetSystemDirectoryA(buffer, 300)
        dirs.insert(0, buffer.value.decode('mbcs'))
    except:
        pass
    for dir in dirs:
        try:
            pipe = os.popen(os.path.join(dir, 'ipconfig') + ' /all')
        except IOError:
            continue
        for line in pipe:
            value = line.split(':')[-1].strip().lower()
            if re.match('([0-9a-f][0-9a-f]-){5}[0-9a-f][0-9a-f]', value):
                return int(value.replace('-', ''), 16)

def _netbios_getnode():
    """Get the hardware address on Windows using NetBIOS calls.
    See http://support.microsoft.com/kb/118623 for details."""
    import win32wnet, netbios
    ncb = netbios.NCB()
    ncb.Command = netbios.NCBENUM
    ncb.Buffer = adapters = netbios.LANA_ENUM()
    adapters._pack()
    if win32wnet.Netbios(ncb) != 0:
        return
    adapters._unpack()
    for i in range(adapters.length):
        ncb.Reset()
        ncb.Command = netbios.NCBRESET
        ncb.Lana_num = ord(adapters.lana[i])
        if win32wnet.Netbios(ncb) != 0:
            continue
        ncb.Reset()
        ncb.Command = netbios.NCBASTAT
        ncb.Lana_num = ord(adapters.lana[i])
        ncb.Callname = '*'.ljust(16)
        ncb.Buffer = status = netbios.ADAPTER_STATUS()
        if win32wnet.Netbios(ncb) != 0:
            continue
        status._unpack()
        bytes = map(ord, status.adapter_address)
        return ((bytes[0]<<40L) + (bytes[1]<<32L) + (bytes[2]<<24L) +
                (bytes[3]<<16L) + (bytes[4]<<8L) + bytes[5])

# Thanks to Thomas Heller for ctypes and for his help with its use here.

# If ctypes is available, use it to find system routines for UUID generation.
_uuid_generate_random = _uuid_generate_time = _UuidCreate = None
try:
    import ctypes, ctypes.util
    _buffer = ctypes.create_string_buffer(16)

    # The uuid_generate_* routines are provided by libuuid on at least
    # Linux and FreeBSD, and provided by libc on Mac OS X.
    for libname in ['uuid', 'c']:
        try:
            lib = ctypes.CDLL(ctypes.util.find_library(libname))
        except:
            continue
        if hasattr(lib, 'uuid_generate_random'):
            _uuid_generate_random = lib.uuid_generate_random
        if hasattr(lib, 'uuid_generate_time'):
            _uuid_generate_time = lib.uuid_generate_time

    # On Windows prior to 2000, UuidCreate gives a UUID containing the
    # hardware address.  On Windows 2000 and later, UuidCreate makes a
    # random UUID and UuidCreateSequential gives a UUID containing the
    # hardware address.  These routines are provided by the RPC runtime.
    try:
        lib = ctypes.windll.rpcrt4
    except:
        lib = None
    _UuidCreate = getattr(lib, 'UuidCreateSequential',
                          getattr(lib, 'UuidCreate', None))
except:
    pass

def _unixdll_getnode():
    """Get the hardware address on Unix using ctypes."""
    _uuid_generate_time(_buffer)
    return UUID(bytes=_buffer.raw).node

def _windll_getnode():
    """Get the hardware address on Windows using ctypes."""
    if _UuidCreate(_buffer) == 0:
        return UUID(bytes=_buffer.raw).node

def _random_getnode():
    """Get a random node ID, with eighth bit set as suggested by RFC 4122."""
    import random
    return random.randrange(0, 1<<48L) | 0x010000000000L

_node = None

def getnode():
    """Get the hardware address as a 48-bit integer.  The first time this
    runs, it may launch a separate program, which could be quite slow.  If
    all attempts to obtain the hardware address fail, we choose a random
    48-bit number with its eighth bit set to 1 as recommended in RFC 4122."""

    global _node
    if _node is not None:
        return _node

    import sys
    if sys.platform == 'win32':
        getters = [_windll_getnode, _netbios_getnode, _ipconfig_getnode]
    else:
        getters = [_unixdll_getnode, _ifconfig_getnode]

    for getter in getters + [_random_getnode]:
        try:
            _node = getter()
        except:
            continue
        if _node is not None:
            return _node

def uuid1(node=None, clock_seq=None):
    """Generate a UUID from a host ID, sequence number, and the current time.
    If 'node' is not given, getnode() is used to obtain the hardware
    address.  If 'clock_seq' is given, it is used as the sequence number;
    otherwise a random 14-bit sequence number is chosen."""

    # When the system provides a version-1 UUID generator, use it (but don't
    # use UuidCreate here because its UUIDs don't conform to RFC 4122).
    if _uuid_generate_time and node is clock_seq is None:
        _uuid_generate_time(_buffer)
        return UUID(bytes=_buffer.raw)

    import time
    nanoseconds = int(time.time() * 1e9)
    # 0x01b21dd213814000 is the number of 100-ns intervals between the
    # UUID epoch 1582-10-15 00:00:00 and the Unix epoch 1970-01-01 00:00:00.
    timestamp = int(nanoseconds/100) + 0x01b21dd213814000L
    if clock_seq is None:
        import random
        clock_seq = random.randrange(1<<14L) # instead of stable storage
    time_low = timestamp & 0xffffffffL
    time_mid = (timestamp >> 32L) & 0xffffL
    time_hi_version = (timestamp >> 48L) & 0x0fffL
    clock_seq_low = clock_seq & 0xffL
    clock_seq_hi_variant = (clock_seq >> 8L) & 0x3fL
    if node is None:
        node = getnode()
    return UUID(fields=(time_low, time_mid, time_hi_version,
                        clock_seq_hi_variant, clock_seq_low, node), version=1)

def uuid3(namespace, name):
    """Generate a UUID from the MD5 hash of a namespace UUID and a name."""
    import md5
    hash = md5.md5(namespace.bytes + name).digest()
    return UUID(bytes=hash[:16], version=3)

def uuid4():
    """Generate a random UUID."""

    # When the system provides a version-4 UUID generator, use it.
    if _uuid_generate_random:
        _uuid_generate_random(_buffer)
        return UUID(bytes=_buffer.raw)

    # Otherwise, get randomness from urandom or the 'random' module.
    try:
        import os
        return UUID(bytes=os.urandom(16), version=4)
    except:
        import random
        bytes = [chr(random.randrange(256)) for i in range(16)]
        return UUID(bytes=bytes, version=4)

def uuid5(namespace, name):
    """Generate a UUID from the SHA-1 hash of a namespace UUID and a name."""
    import sha
    hash = sha.sha(namespace.bytes + name).digest()
    return UUID(bytes=hash[:16], version=5)

# The following standard UUIDs are for use with uuid3() or uuid5().

NAMESPACE_DNS = UUID('6ba7b810-9dad-11d1-80b4-00c04fd430c8')
NAMESPACE_URL = UUID('6ba7b811-9dad-11d1-80b4-00c04fd430c8')
NAMESPACE_OID = UUID('6ba7b812-9dad-11d1-80b4-00c04fd430c8')
NAMESPACE_X500 = UUID('6ba7b814-9dad-11d1-80b4-00c04fd430c8')



class Table(object):
    def __init__(self, id="", name="", current_pos_id="", status="", tab_id="", business_period="", sector=""):
        # type: (str, str, int, int, bool, int, str) -> None

        self.id = id
        self.name = name
        self.current_pos_id = current_pos_id
        self.status = status
        self.tab_id = tab_id
        self.business_period = business_period
        self.sector = sector



from abc import ABCMeta, abstractmethod

from typing import List, Dict  # noqa


class I18nRepository(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def internationalize(self, labels):
        # type: (List[str]) -> Dict[str, str]
        raise NotImplementedError()

    @abstractmethod
    def get_date_format(self):
        # type: () -> str
        raise NotImplementedError()

    @abstractmethod
    def get_date_time_format(self):
        # type: () -> str
        raise NotImplementedError()

    @abstractmethod
    def get_decimal_separator(self):
        # type: () -> str
        raise NotImplementedError()

    @abstractmethod
    def get_thousands_separator(self):
        # type: () -> str
        raise NotImplementedError()

    @abstractmethod
    def get_currency_symbol(self):
        # type: () -> str
        raise NotImplementedError()

    @abstractmethod
    def get_currency_symbol_position(self):
        # type: () -> str
        raise NotImplementedError()

    @abstractmethod
    def get_decimal_places(self):
        # type: () -> str
        raise NotImplementedError()



from logging import getLogger
from typing import Any, Dict

from nperipherals_core.model.device import WriterDevice

logger = getLogger(__name__)


class NetworkPrinterDevice(WriterDevice):
    def __init__(self, device_configurations):
        # type: (Dict[unicode, Any]) -> None

        self.name = device_configurations["Name"]  # type: unicode
        self.host = device_configurations["Host"]  # type: unicode
        self.port = device_configurations["Port"]  # type: int

    def get_name(self):
        # type: () -> unicode

        return self.name

    def initialize(self):
        # type: () -> None

        raise NotImplementedError()

    def connect(self):
        # type: () -> None

        raise NotImplementedError()

    def disconnect(self):
        # type: () -> None

        raise NotImplementedError()

    def write(self, data):
        # type: (unicode) -> None

        raise NotImplementedError()



import os
import json
import logging
from logging import Logger

from helper import config_logger
from typing import Optional

COMPONENT_NAME = "nperipherals"


def configure_logger(component_name, service_name=None, root_level=logging.DEBUG):
    # type: (str, Optional[str], int) -> Logger

    log_path = os.environ["LOG_PATH"]

    if os.path.isfile(log_path) and log_path.endswith(".json"):
        log_path = _update_log_name(component_name, service_name, log_path)

    config_logger(log_path, component_name, root_level=root_level, max_files=10, max_size=10485760)
    return logging.getLogger(component_name)


def _update_log_name(component_name, service_name, log_path):
    # type: (str, str, str) -> str

    with open(log_path, "rb") as f:
        log_config = json.loads(f.read().decode("utf-8"))
        file_name = log_config.get("handlers", {}).get("default", {}).get("filename", None)

        if not file_name or "$SERVICE_NAME" not in file_name:
            return log_path

        log_name = "{}_{}".format(component_name, service_name) if service_name else component_name
        file_name = file_name.replace("$SERVICE_NAME", log_name)
        log_config["handlers"]["default"]["filename"] = file_name
        log_path = log_config

    return log_path



from ._GetBenefitApplier import \
    fill_benefit_appliers_repositories, \
    find_benefit_applier, \
    find_benefit_id_by_custom_property, \
    find_order_benefits_id, \
    get_originator_pos_id



# encoding: utf-8
import json
import datetime
import time
from cStringIO import StringIO

from utils import add_amount_line, add_key_value_line, break_line, report_default_header, empty_line
from domain import TransferType
from old_helper import convert_from_utf_to_localtime

DATE_TIME_FMT = "%d/%m/%Y %H:%M:%S"


class TransfersReport(object):
    def __init__(self, account_repository, store_id):
        self.account_repository = account_repository
        self.store_id = store_id

    def generate_transfers_report(self, pos_id, operator_id, transfer_type, amount, period, envelope_number,
                                  print_operator_copy):
        print_operator_copy = print_operator_copy.lower() == 'true'
        report = StringIO()
        title = "Sangria" if int(transfer_type) == TransferType.TRANSFER_CASH_OUT else "Suprimento"
        title += " (Via Gerente)" if not print_operator_copy else " (Via Operador)"
        current_datetime = time.strftime(DATE_TIME_FMT)

        report.write(report_default_header(title, int(pos_id), operator_id, period, current_datetime, self.store_id))
        self._transfers_report_body(amount, envelope_number, "", report)

        return report.getvalue()

    def generate_transfers_report_by_period(self, pos_id, operator_id, transfer_type, period):
        transfer_type = int(transfer_type)
        transfers = self.account_repository.get_transfers_by_period(period, transfer_type)
        reports = []
        title = "Sangria" if transfer_type == TransferType.TRANSFER_CASH_OUT else "Suprimento"

        for transfer in transfers:
            report = StringIO()

            current_datetime = datetime.datetime.strptime(transfer.operation_date, "%Y-%m-%d %H:%M:%S")
            current_datetime = convert_from_utf_to_localtime(current_datetime).strftime('%d-%m-%Y %H:%M:%S')

            report.write(report_default_header(title, int(pos_id), operator_id, period, current_datetime, self.store_id))
            envelope_number = transfer.glaccount.split("envelope\": \"")[1].split("\"")[0]
            manager = transfer.glaccount.split("manager\": \"")[1].split("\"")[0]
            self._transfers_report_body(transfer.amount, envelope_number, manager, report)

            reports.append({current_datetime: report.getvalue()})

        return json.dumps(reports, default=lambda o: o.__dict__, sort_keys=True)

    def _transfers_report_body(self, amount, envelope_number, manager, report):
        self._write_cash_in_and_cash_out(amount, envelope_number, manager, report)

    @staticmethod
    def _write_cash_in_and_cash_out(amount, envelope_number, manager, report):
        if amount != "":
            report.write(break_line())
            report.write(add_amount_line("Valor", None, amount))
        if envelope_number != "":
            report.write(add_key_value_line("Numero Envelope..: ", envelope_number))
        if manager != "":
            report.write(add_key_value_line("Autorizado por...: ", manager))



import logging
import os

import cfgtools
from helper import import_pydevd, config_logger
from mbcontextmessagehandler import MbContextMessageBus, MbContextMessageHandler
from messagehandlerbuilder import NKDSCtrlMessageHandlerBuilder
from model import Kds, KdsView
from msgbus import MBEasyContext
from typing import List


def main():
    import_pydevd(os.environ["LOADERCFG"], 9135, False)
    config_logger(os.environ["LOADERCFG"], "NKDSCTRL")

    logger = logging.getLogger("NKDSCTRL")
    logger.setLevel(logging.DEBUG)
    proxy_comp_name = "KdsController"

    mb_context = MBEasyContext("NKDSCTRL")
    message_bus = MbContextMessageBus(mb_context)

    message_handler = MbContextMessageHandler(message_bus, "NKDSCTRL", "NKDSCTRL", "ProductionSystem", None)
    config = cfgtools.read(os.environ["LOADERCFG"])
    kds_views = get_kds_views(config)
    kds_list = get_kds_list(config, kds_views)

    message_handler_builder = NKDSCtrlMessageHandlerBuilder(proxy_comp_name, kds_list, message_bus, logger)
    message_handler.event_handler_builder = message_handler_builder

    return message_handler.handle_events()


def get_kds_views(config):
    # type: (cfgtools.Group) -> List[KdsView]
    views = []
    view_config = config.find_group("Views")
    if not view_config:
        return []

    for view_config_group in view_config.groups:
        views.append(KdsView(view_config_group))

    return views


def get_kds_list(config, kds_views):
    # type: (cfgtools.Group, List[KdsView]) -> List[Kds]
    kds_list = []
    kds_config = config.find_group("KDS")
    if not kds_config:
        return []
    for kds_config_group in kds_config.groups:
        kds_list.append(Kds(kds_config_group, kds_views))

    return kds_list



from production.box import CustomPropertyValueFilterBox
from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder


class CustomPropertyValueFilterBoxBuilder(BoxBuilder):
    def __init__(self, loggers):
        super(CustomPropertyValueFilterBoxBuilder, self).__init__(loggers)

    def build(self, config):
        # type: (BoxConfiguration) -> CustomPropertyValueFilterBox
        return CustomPropertyValueFilterBox(config.name,
                                            config.sons,
                                            self.get_extra(config, "PropertyName", None),
                                            self.get_extra(config, "AllowedValues", None),
                                            self.get_extra(config, "ExcludedValues", None),
                                            self.get_logger(config))



import os
from abc import ABCMeta

from msgbus import MBEasyContext
from persistence import Connection, Driver
from typing import Callable, Any, List


class BaseRepository(object):
    __metaclass__ = ABCMeta

    def __init__(self, mb_context):
        # type: (MBEasyContext) -> None
        self.mb_context = mb_context

    def execute_with_connection(self, method, db_name=None, service_name=None):
        # type: (Callable[[Connection], Any], int, str) -> Any
        conn = None
        try:
            conn = Driver().open(self.mb_context,
                                 dbname=str(db_name) if db_name is not None else None,
                                 service_name=service_name)
            return method(conn)
        finally:
            if conn is not None:
                conn.close()

    def execute_with_transaction(self, method, db_name=None, service_name=None):
        # type: (Callable[[Connection], Any], int, str) -> Any
        def transaction_method(conn):
            # type: (Connection) -> Any

            conn.transaction_start()
            try:
                conn.query("BEGIN TRANSACTION")
                ret = method(conn)
                conn.query("COMMIT")
                return ret
            except:
                conn.query("ROLLBACK")
                raise
            finally:
                conn.transaction_end()

        return self.execute_with_connection(transaction_method, db_name=db_name, service_name=service_name)

    def execute_in_all_databases(self, method, pos_list):
        # type: (Callable[[Connection], Any], List[int]) -> Any
        ret = []
        for pos in pos_list:
            conn = None
            try:
                conn = Driver().open(self.mb_context, dbname=str(pos))
                ret.append(method(conn))
            finally:
                if conn is not None:
                    conn.close()

        return ret

    def execute_in_all_databases_returning_flat_list(self, method, pos_list):
        # type: (Callable[[Connection], Any], List[int]) -> Any
        inner_ret = self.execute_in_all_databases(method, pos_list)

        ret = []
        for inner_list in inner_ret:
            ret.extend(inner_list)

        return ret


def find_value_with_vars(cfg, path, default_value=None):
    value = cfg.find_value(path, default_value)
    if value is None:
        return value

    for variable_name in os.environ:
        variable_format = "{{{}}}".format(variable_name)
        if variable_format in value:
            value = value.replace(variable_format, os.environ[variable_name])
            if variable_name in ("BUNDLEDIR", "HVDATADIR"):
                value = value.replace("/./", os.sep).replace("//", os.sep).replace("/", os.sep)

    return value



import base64
import json

from constants import PAPER_PART_CUT
from msgbus import TK_PRN_PRINT, FM_PARAM
from sysactions import get_storewide_config, get_model, get_current_order, get_used_service, show_confirmation, \
    get_podtype


class PrintReceipt(object):
    def __init__(self, mb_context, logger):
        self.mb_context = mb_context
        self.logger = logger

        merchant_config = "Store.PrintMerchantReceipt"
        self.print_merchant_receipt = get_storewide_config(merchant_config, defval="False").lower() == "true"
        ask_customer_config = "Store.AskPrintCustomerReceipt"
        self.ask_print_customer_receipt = get_storewide_config(ask_customer_config, defval="True").lower() == "true"
        customer_config = "Store.PrintCustomerReceipt"
        self.print_customer_receipt = get_storewide_config(customer_config, defval="False").lower() == "true"

    def print_tef_receipts(self, pos_id):
        try:
            model = get_model(pos_id)
            tenders = get_current_order(model).findall("TenderHistory/Tender")
            electronic_tenders = []
            for tender in tenders:
                if int(tender.get("eletronicType")) not in (0, 9):
                    electronic_tenders.append(tender)

            if len(electronic_tenders) == 0:
                return

            current_printer = get_used_service(model, "printer")

            pod_type = get_podtype(model)
            if self.print_merchant_receipt and pod_type != "TT":
                self._print_receipt(current_printer, electronic_tenders, "ReceiptMerchant")

            if self._has_customer_receipt(electronic_tenders):
                if self.ask_print_customer_receipt:
                    response = show_confirmation(pos_id, "$WANT_TO_PRINT_CUSTOMER_RECEIPT")
                    if response:
                        self._print_receipt(current_printer, electronic_tenders, "ReceiptCustomer")
                elif self.print_customer_receipt:
                    self._print_receipt(current_printer, electronic_tenders, "ReceiptCustomer")

        except Exception as _:
            self.logger.exception("Error printing TEF receipts")

    def _print_receipt(self, current_printer, electronic_tenders, receipt_type):
        for tender in electronic_tenders:
            receipt_data = json.loads(tender.get("tenderDetail"))[receipt_type]
            if receipt_data != "":
                receipt = base64.b64decode(receipt_data)
                receipt += PAPER_PART_CUT
                self.mb_context.MB_EasySendMessage(current_printer, TK_PRN_PRINT, FM_PARAM, receipt, 30000000)

    def _has_customer_receipt(self, electronic_tenders):
        for tender in electronic_tenders:
            receipt_data = json.loads(tender.get("tenderDetail"))["ReceiptCustomer"]
            if receipt_data != "":
                return True
        return False



from __future__ import absolute_import
import sys

from future.utils import PY2, PY26
__future_module__ = True

from collections import *

if PY2:
    from UserDict import UserDict
    from UserList import UserList
    from UserString import UserString

if PY26:
    from future.backports.misc import OrderedDict, Counter

if sys.version_info < (3, 3):
    from future.backports.misc import ChainMap, _count_elements



from xml.etree import cElementTree as eTree


def get_user_data(model):
    # type: (eTree) -> (str, eTree.XML)

    operator = model.find("Operator")
    if operator is None:
        return 0, None

    user_id = operator.get("id")
    user_level = int(operator.get("level", 0))

    return user_level, user_id



# -*- coding: utf-8 -*-

import datetime
import json
import sys

import iso8601
import six
from chatservice.model import Message, MessageFrom
from chatservice.customexception import InvalidJsonException
from typing import List, Dict


class MessageParser(object):
    
    def parse(self, input_json):
        # type: (unicode) -> Message
        
        try:
            message_json = json.loads(input_json)
        except ValueError as ex:
            raise six.reraise(InvalidJsonException, InvalidJsonException(ex.message, input_json), sys.exc_info()[2])

        try:
            self._validate_json_structure(message_json)
            return self._parse_message(message_json)
        except InvalidJsonException as ex:
            six.reraise(InvalidJsonException, InvalidJsonException(ex.message, input_json), sys.exc_info()[2])

    def parse_sac_messages(self, input_json):
        # type: (unicode) -> List[Message]
        
        try:
            messages_json = json.loads(input_json)
        except ValueError as ex:
            raise six.reraise(InvalidJsonException, InvalidJsonException(ex.message, input_json), sys.exc_info()[2])

        try:
            if not isinstance(messages_json, list):
                raise InvalidJsonException("The json should be a list")

            messages = []
            for message_json in messages_json:
                self._validate_sac_json_structure(message_json)
                message = self._parse_sac_message(message_json)
                messages.append(message)

            return messages
        except InvalidJsonException as ex:
            six.reraise(InvalidJsonException, InvalidJsonException(ex.message, input_json), sys.exc_info()[2])

    def parse_sac_ack_messages(self, input_json):
        # type: (unicode) -> List[Message]
        
        try:
            messages_json = json.loads(input_json)
        except ValueError as ex:
            raise six.reraise(InvalidJsonException, InvalidJsonException(ex.message, input_json), sys.exc_info()[2])

        try:
            if not isinstance(messages_json, list):
                raise InvalidJsonException("The json should be a list")

            messages = []
            for message_json in messages_json:
                self._validate_sac_ack_json_structure(message_json)
                message = self._parse_sac_ack_message(message_json)
                messages.append(message)

            return messages
        except InvalidJsonException as ex:
            six.reraise(InvalidJsonException, InvalidJsonException(ex.message, input_json), sys.exc_info()[2])

    @staticmethod
    def _validate_json_structure(message_json):
        # type: (Dict) -> None
        
        if "text" not in message_json:
            raise InvalidJsonException("Json without 'text' tag")

        if message_json["text"] == "":
            raise InvalidJsonException("The 'text' field cannot be empty")

    @staticmethod
    def _validate_sac_json_structure(message_json):
        # type: (Dict) -> None
        
        if "id" not in message_json:
            raise InvalidJsonException("Json without 'id' tag")

        if "text" not in message_json:
            raise InvalidJsonException("Json without 'text' tag")

        if "createdTime" not in message_json:
            raise InvalidJsonException("Json without 'createdTime' tag")

        try:
            iso8601.parse_date(message_json["createdTime"])
        except iso8601.ParseError:
            raise InvalidJsonException("Invalid 'createTime': " + message_json["createTime"])

    @staticmethod
    def _validate_sac_ack_json_structure(message_json):
        # type: (Dict) -> None
        
        if "id" not in message_json:
            raise InvalidJsonException("Json without 'id' tag")

        if "receivedTime" not in message_json:
            raise InvalidJsonException("Json without 'receivedTime' tag")

        try:
            iso8601.parse_date(message_json["receivedTime"])
        except iso8601.ParseError:
            raise InvalidJsonException("Invalid 'createTime': " + message_json["receivedTime"])

    @staticmethod
    def _parse_message(message_json):
        # type: (Dict) -> Message
        
        message = Message()
        message.id = None
        message.text = message_json.get("text")
        message.message_from = MessageFrom.STORE
        message.created_time = datetime.datetime.utcnow()
        message.received_time = None
        message.server_id = None

        return message

    @staticmethod
    def _parse_sac_message(message_json):
        # type: (Dict) -> Message
        
        message = Message()
        message.id = None
        message.text = message_json["text"]
        message.server_id = message_json["id"]
        message.message_from = MessageFrom.SAC
        message.created_time = iso8601.parse_date(message_json["createdTime"])
        message.received_time = datetime.datetime.utcnow()

        return message

    @staticmethod
    def _parse_sac_ack_message(message_json):
        # type: (Dict) -> Message
        
        message = Message()
        message.id = message_json["id"]
        message.text = None
        message.server_id = None
        message.message_from = MessageFrom.SAC
        message.created_time = None
        message.received_time = iso8601.parse_date(message_json["receivedTime"])

        return message



from unittest import TestCase

import simplejson
from simplejson.compat import text_type

# Tests for issue demonstrated in https://github.com/simplejson/simplejson/issues/144
class WonkyTextSubclass(text_type):
    def __getslice__(self, start, end):
        return self.__class__('not what you wanted!')

class TestStrSubclass(TestCase):
    def test_dump_load(self):
        for s in ['', '"hello"', 'text', u'\u005c']:
            self.assertEqual(
                s,
                simplejson.loads(simplejson.dumps(WonkyTextSubclass(s))))

            self.assertEqual(
                s,
                simplejson.loads(simplejson.dumps(WonkyTextSubclass(s),
                                                  ensure_ascii=False)))



import json

from messageprocessorutil import UuidMessageProcessor
from messagebus import NakMessage, AckMessage
from production.model import ProductionModelJsonEncoder

from production.repository import ProductRepository
from production.interactor import GetCoursesInteractor


class GetCoursesProcessor(UuidMessageProcessor):
    def __init__(self, product_repository):
        # type: (ProductRepository) -> None

        super(GetCoursesProcessor, self).__init__()
        self.interactor = GetCoursesInteractor(product_repository)

    def parse_data(self, _):
        return ''

    def call_business(self, obj):
        return self.interactor.execute()

    def format_response(self, message_bus, message, event, input_obj, result):
        result = json.dumps(result, encoding="utf-8", cls=ProductionModelJsonEncoder)
        message_bus.reply_message(message, AckMessage(result))

    def format_exception(self, message_bus, message, event, input_obj, exception):
        message_bus.reply_message(message, NakMessage())

    def format_parse_exception(self, message_bus, message, event, data, exception):
        message_bus.reply_message(message, NakMessage())



import sysactions
from actions.kiosk import do_void_kiosk_order
from actions.void.do_void_order import do_void_order


@sysactions.action
def handle_void_order(pos_id, void_reason=None, fiscal_void="false", order_id="", show_cancel_message="true"):
    try:
        model = sysactions.get_model(pos_id)
        pod_type = sysactions.get_podtype(model)
        is_totem_order = pod_type == "TT"
        if is_totem_order:
            return do_void_kiosk_order(pos_id)
        return do_void_order(pos_id, void_reason, fiscal_void, order_id, show_cancel_message == "true")
    except BaseException as _:
        return False



#
# The Python Imaging Library
# $Id$
#
# path interface
#
# History:
# 1996-11-04 fl   Created
# 2002-04-14 fl   Added documentation stub class
#
# Copyright (c) Secret Labs AB 1997.
# Copyright (c) Fredrik Lundh 1996.
#
# See the README file for information on usage and redistribution.
#

from . import Image

Path = Image.core.path



from production.command import CommandType
from production.box import ViewBox
from production.model.request import RefreshViewRequest
from typing import List

from ._CommandProcessor import CommandProcessor


class RefreshViewProcessor(CommandProcessor):
    def __init__(self, view_boxes):
        # type: (List[ViewBox]) -> None

        super(RefreshViewProcessor, self).__init__(view_boxes)

    def parse_data(self, data):
        return RefreshViewRequest(data.split('\0')[0])

    def get_command_type(self):
        return CommandType.refresh_view



from ._HealthCheckProcessor import HealthCheckProcessor
from ._PrintProcessor import PrintProcessor
from ._CashDrawerOpenProcessor import CashDrawerOpenProcessor
from ._CashDrawerStatusProcessor import CashDrawerStatusProcessor
from ._PrinterStatusProcessor import PrinterStatusProcessor



from application.service import BenefitService
from application.model import Configurations
from typing import List


class RemoveBenefitInteractor(object):

    def __init__(self, configs, benefit_service):
        # type: (Configurations, BenefitService) -> None

        self.configs = configs
        self.benefit_service = benefit_service

    def remove(self, pos_id, benefit_keys):
        # type: (int, List) -> None

        self.benefit_service.remove_benefits(pos_id, benefit_keys)
        self.benefit_service.unset_applied_benefit(pos_id, ["IN_PROGRESS", "TOTALED"])
        self.benefit_service.apply_pending_benefits(pos_id)



# -*- coding: utf-8 -*-
import bustoken
import logging
import cfgtools
import os

from mw_helper import show_filtered_list_box_dialog
from systools import sys_log_error
from msgbus import MBEasyContext, MBException, FM_PARAM, TK_SYS_ACK
from sysactions import action, show_listbox, show_keyboard, show_info_message, show_confirmation, show_messagebox, get_storewide_config
from userhelper import UserHelper

from actions.util import get_user_level

mbcontext = None  # type: MBEasyContext
logger = logging.getLogger("manager.users")

config = cfgtools.read(os.environ["LOADERCFG"])
users_with_blocked_changes = config.find_values("UsersManager.BlockedChanges")

# noinspection PyUnusedLocal


@action
def change_user_password(posid, auth_data=None, *args):
    with UserHelper(mbcontext) as user_helper:
        users = user_helper.get_all_users()
        formatted_users = []
        for user in sorted(users, key=lambda x: int(x)):
            formatted_users.append("{} - {} - {}".format(user, users[user][0], users[user][1]))
        index = show_filtered_list_box_dialog(posid, formatted_users, "ID|Nome|Level")
        if index is None:
            return

        selected_user = sorted(users, key=lambda x: int(x))[index]
        user_alter = user_helper.get_level_user(selected_user)

        if selected_user in users_with_blocked_changes:
            show_messagebox(posid, "$CANNOT_CHANGE_PASSWORD")
            return

        if auth_data:
            authorization_level = get_user_level(auth_data[0])
            if int(authorization_level) < int(user_alter.get('level')):
                show_messagebox(posid, "$CANNOT_CHANGE_PASSWORD")
                return

        options = ("SENHA", "DIGITAL")
        type_box = show_messagebox(posid, "$CHOOSE_AUTHENTICATION_TYPE", "$INFORMATION", buttons="|".join(options))

        bt_pressed = options[type_box]

        if bt_pressed == "SENHA":
            valid = False
            passwd1 = None
            while not valid:
                passwd1 = show_keyboard(posid, "$ENTER_NEW_PASSWORD", title="$NEW_PASSWORD", mask="INTEGER", is_password=True, numpad=True)

                if passwd1 is None:
                    return

                if passwd1 in (None, "", "."):
                    show_messagebox(posid, "$SE_INVALIDPASSWD")
                    continue

                passwd2 = show_keyboard(posid, "$REPEAT_PASSWORD", title="$REPEAT_PASSWORD", mask="INTEGER", is_password=True, numpad=True)

                if passwd2 is None:
                    return

                if passwd1 == passwd2:
                    valid = True
                else:
                    show_messagebox(posid, "$PASSWORDS_DONT_MATCH")

            try:
                sucesso = user_helper.change_pass(selected_user, passwd1)
            except Exception as ex:
                sys_log_error("Exceção ao alterar a senha - Exception: " + str(ex))
                logger.exception("Exceção ao alterar a senha")
                show_messagebox(posid, "$ERROR_CHANGING_PASSWORD")
                return

            if sucesso:
                show_messagebox(posid, "$PASSWORD_CHANGED_SUCCESSFULLY")
            else:
                show_messagebox(posid, "$ERROR_CHANGING_PASSWORD")

        elif bt_pressed == "DIGITAL":
            # Checa se temos um leitor de impressao digital
            try:
                ret = mbcontext.MB_EasySendMessage("FingerPrintReader{0}".format(posid), bustoken.TK_FPR_OK)
                if ret.token == TK_SYS_ACK:
                    # Temos o leitor e ele esta operacional, vamos pedir as impressoes do usuario
                    ret = mbcontext.MB_EasySendMessage("FingerPrintReader{0}".format(posid), bustoken.TK_FPR_RE_ENROLL_USER, format=FM_PARAM, data="%s;%s" % (selected_user, posid))
                    if ret.token == bustoken.TK_FPR_FINGER_PRINT_ALREADY_REGISTERED:
                        show_messagebox(posid, "$FINGERPRINT_ALREADY_FOR_USER")
                        return
                    elif ret.token != TK_SYS_ACK:
                        # Erro cadastrando impressao digital
                        show_messagebox(posid, "$ERROR_REGISTERING_FINGERPRINT")
                        return
                    else:
                        show_messagebox(posid, "$FINGERPRINT_CHANGED_SUCCESSFULLY")
                else:
                    show_messagebox(posid, "$READER_FINGERPRINT_NOT_FOUND")
            except MBException as e:
                if e.errorcode == 2:  # NOT_FOUND, servico de FingerPrint nao disponivel
                    show_messagebox(posid, "$READER_FINGERPRINT_NOT_FOUND")
                else:
                    show_messagebox(posid, "$ERROR_CHECKING_READE_FINGERPRINT")


@action
def createuser(pos_id, *args):
    with UserHelper(mbcontext) as user_helper:
        userid = show_keyboard(pos_id, "$ENTER_ID_FOR_NEW_USER", mask="INTEGER", title="ID", numpad=True)
        if not userid:
            return
        if len(userid) > 8:
            show_info_message(pos_id, "$INVALID_ID", msgtype="error")
            return

        user = user_helper.get_user_by_id(userid)
        if user:
            show_info_message(pos_id, "$USER_ID_ALREADY", msgtype="error")
            return

        username = show_keyboard(pos_id, "$ENTER_NAME_NEW_USER", title="Nome")
        username = str.strip(username) if username else ""
        if username in "":
            show_info_message(pos_id, "$ENTER_NAME_NEW_USER", msgtype="error")
            return

        matching = [s for s in ['|', '-', '+', '[', ']', ';', '/', ',', '.', '!', '@', '#', '^', '$', '%', '&', '*', '(', ')', '_', '=', '{', '}', ':', '?', '<', '>'] if s in username]
        if matching:
            show_info_message(pos_id, "$PROHIBITED_CHARACTERS".format_map(str(matching)), msgtype="error")
            return

        valid = False
        passwd1 = None
        while not valid:
            passwd1 = show_keyboard(pos_id, "$ENTER_PASSWORD_TO_REGISTER", title="$ENTER_PASSWORD_TO_REGISTER", mask="INTEGER", is_password=True, numpad=True)
            if passwd1 in (None, "", "."):
                show_info_message(pos_id, "$INVALID_PASSWORD", msgtype="error")
                return
            passwd2 = show_keyboard(pos_id, "$REPEAT_PASSWORD", title="$REPEAT_PASSWORD", mask="INTEGER", is_password=True, numpad=True)
            if passwd1 == passwd2:
                valid = True
            else:
                show_info_message(pos_id, "$PASSWORDS_DONT_MATCH", msgtype="error")

        admin = show_confirmation(pos_id, message="$MAKE_USER_SUPERVISOR", buttons="$YES|$NO")

        # Checa se temos um leitor de impressao digital
        fmd_data = None

        try:
            ret = mbcontext.MB_EasySendMessage("FingerPrintReader{0}".format(pos_id), bustoken.TK_FPR_OK)
            if ret.token == TK_SYS_ACK:
                # Temos o leitor e ele esta operacional, vamos pedir as impressoes do usuario
                ret = mbcontext.MB_EasySendMessage("FingerPrintReader{0}".format(pos_id), bustoken.TK_FPR_ENROLL_USER, format=FM_PARAM, data="%s;%s" % (userid, pos_id))

                if ret.token == bustoken.TK_FPR_FINGER_PRINT_ALREADY_REGISTERED:
                    show_info_message(pos_id, "$FINGERPRINT_ALREADY_FOR_USER", msgtype="error")
                    return
                elif ret.token != TK_SYS_ACK:
                    # Erro cadastrando impressao digital
                    show_info_message(pos_id, "$ERROR_REGISTERING_FINGERPRINT", msgtype="error")
                    return
                else:
                    fmd_data = ret.data
        except MBException as _:
            # NOT_FOUND, FingerPrint service not available
            pass
        except Exception as ex:
            sys_log_error("Exceção ao cadastrar digital - Exception: " + str(ex))
            logger.exception("Exceção ao cadastrar digital")
            show_info_message(pos_id, "$ERROR_REGISTERING_FINGERPRINT", msgtype="error")

        try:
            sucesso = user_helper.add_user(userid, username, passwd1, admin, fmd_data)
        except Exception as ex:
            sys_log_error("Exceção ao cadastrar usuário - Exception: " + str(ex))
            logger.exception("Exceção ao cadastrar usuário")
            show_info_message(pos_id, "$ERROR_INSERT_NEW_USER", msgtype="error")
            return

        if sucesso:
            show_info_message(pos_id, "$SUCCESS_INSERT_NEW_USER", msgtype="info")
        else:
            show_info_message(pos_id, "$ERROR_INSERT_NEW_USER", msgtype="error")


@action
def removeuser(pos_id, *args):
    with UserHelper(mbcontext) as user_helper:
        # lista de usuarios
        users = user_helper.get_users_by_level("0")
        # lista de supervisores
        users.update(user_helper.get_users_by_level("20"))

        users_formatted = ["%s - %s" % (i, users.get(i)) for i in users]
        index = show_listbox(pos_id, users_formatted, message="$USERS", buttons="$OK")

        if users.keys()[index] in users_with_blocked_changes:
            show_info_message(pos_id, "$USER_CANNOT_BE_REMOVED", msgtype="error")
            return

        confirm = show_confirmation(pos_id, message="$DELETE_USER", buttons="$YES|$NO")

        if confirm:
            # noinspection PyBroadException
            try:
                sucesso = user_helper.remove_user(users.keys()[index])
            except Exception as ex:
                sys_log_error("Exceção removendo usuário - Exception: " + str(ex))
                logger.exception("Exceção removendo usuário")
                show_info_message(pos_id, "$ERROR_DELETE_USER", msgtype="error")
                return

            if sucesso:
                show_info_message(pos_id, "$SUCCESS_DELETE_USER", msgtype="info")
            else:
                show_info_message(pos_id, "$ERROR_DELETE_USER", msgtype="error")
        else:
            show_info_message(pos_id, "$ERROR_DELETE_USER", msgtype="warning")


@action
def inactivateuser(posid, *args):
    with UserHelper(mbcontext) as user_helper:
        # lista de usuarios
        users = user_helper.get_users_by_level("0")
        # lista de supervisores
        users.update(user_helper.get_users_by_level("20"))
        users_formatted = ["%s - %s" % (i, users.get(i)) for i in users]
        index = show_listbox(posid, users_formatted, message="$USERS", buttons="$OK|$CANCEL")
        if index is None:
            return

        if users.keys()[index] in users_with_blocked_changes:
            show_info_message(posid, "$USER_CANNOT_BE_DEACTIVATED", msgtype="error")
            return

        confirm = show_confirmation(posid, message="$CONFIRMS_USER_DISABLEMENT", buttons="$YES|$NO")

        if confirm:
            # noinspection PyBroadException
            try:
                sucesso = user_helper.activate_user(users.keys()[index], op=1)
            except Exception as ex:
                sys_log_error("Exceção ativando usuário - Exception: " + str(ex))
                logger.exception("Exceção ativando usuário")
                show_info_message(posid, "$ERROR_DISABLE_USER", msgtype="error")
                return

            if sucesso:
                show_info_message(posid, "$SUCCESS_DISABLE_USER")
            else:
                show_info_message(posid, "$ERROR_DISABLE_USER", msgtype="error")
        else:
            show_info_message(posid, "$CANCELED_DISABLE_USER", msgtype="warning")


@action
def activateuser(posid, user_level='administrator', *args):
    with UserHelper(mbcontext) as user_helper:
        # lista de usuarios
        users = user_helper.get_inactive_users_by_level("0")
        # lista de supervisores
        users.update(user_helper.get_inactive_users_by_level("20"))
        users_formatted = ["%s - %s" % (i, users.get(i)) for i in users]

        buttons = "$CANCEL"
        allow_reactivate_user = get_storewide_config("Store.AllowSupervisorsReactivateUser", defval="false").lower() == "true"
        if allow_reactivate_user or user_level == 'administrator':
            buttons = "$OK|$CANCEL"

        index = show_listbox(posid, users_formatted, message="$USERS", buttons=buttons)
        if index is None or not allow_reactivate_user:
            return

        if users.keys()[index] in users_with_blocked_changes:
            show_info_message(posid, "$USER_CANNOT_BE_ACTIVATED", msgtype="error")
            return

        confirm = show_confirmation(posid, message="$CONFIRMS_USER_ENABLE", buttons="$YES|$NO")

        if confirm:
            # noinspection PyBroadException
            try:
                sucesso = user_helper.activate_user(users.keys()[index], op=0)
            except Exception as ex:
                sys_log_error("Exceção ativando usuário - Exception: " + str(ex))
                logger.exception("Exceção ativando usuário")
                show_info_message(posid, "$ERROR_ENABLE_USER", msgtype="error")
                return

            if sucesso:
                show_info_message(posid, "$SUCCESS_ENABLE_USER")
            else:
                show_info_message(posid, "$ERROR_ENABLE_USER", msgtype="error")
        else:
            show_info_message(posid, "$CANCELED_ENABLE_USER", msgtype="warning")

@action
def changeperfil(posid, *args):
    with UserHelper(mbcontext) as user_helper:
        options = ("$OPERATOR", "$SUPERVISOR")
        type_box = show_messagebox(posid, "$CHOOSE_PROFILE", "$INFORMATION", buttons="|".join(options))

        level = "20" if type_box else "0"
        level_intended = "0" if "20" == level else "20"
        users = user_helper.get_users_by_level(level)
        list_users = ["%s - %s" % (i, users.get(i)) for i in users]
        index = show_listbox(posid, list_users, message="$USERS")

        if index is None:
            return

        if users.keys()[index] in users_with_blocked_changes:
            show_info_message(posid, "$USER_CANNOT_BE_CHANGED", msgtype="error")
            return

        text_level = {"20": "$CONFIRM_CHANGE_OPERATOR_TO_SUPERVISOR",
                      "0": "$CONFIRM_CHANGE_SUPERVISOR_TO_OPERATOR"}

        confirm = show_confirmation(posid, message=text_level[level_intended], buttons="$YES|$NO")

        if confirm:
            try:
                sucesso = user_helper.change_level_user(users.keys()[index], level_intended)
            except Exception as ex:
                sys_log_error("Exceção ativando usuário - Exception: " + str(ex))
                logger.exception("Exceção ao alterar o perfil do usuário")
                show_info_message(posid, "$ERROR_CHANGE_PROFILE", msgtype="error")
                return

            if sucesso:
                show_info_message(posid, "$SUCCESS_CHANGE_PROFILE")
            else:
                show_info_message(posid, "$ERROR_CHANGE_PROFILE", msgtype="error")
        else:
            show_info_message(posid, "$CANCELED_CHANGE_PROFILE", msgtype="warning")


@action
def list_all_users(pos_id):
    with UserHelper(mbcontext) as user_helper:
        users = user_helper.get_all_users()
        list_users = []
        for user in sorted(users, key=lambda x: int(x)):
            user_info = "{} - {} - {} - {}".format(user,
                                                   users[user][0],
                                                   users[user][1],
                                                   str(not int(users[user][2])))
            list_users.append(user_info)
        title = "Id|Name|Level|Status"
        show_listbox(pos_id, list_users, message=title)


def _info_user_session(posid):
    level, info = None, dict()
    with UserHelper(mbcontext) as user_helper:
        user_session = user_helper.get_user_session(posid)
        try:
            for i in user_session:
                level = {'level': int(user_session[i])}
                info = {x.split('=')[0]: x.split('=')[1] for x in i.split(",")}
            return dict(info, **level)
        except MBException as _:
            return



from basereport import I18n
from report import Reporter, Presenter, Generator

from ._I18nReport import I18nReport


class I18nReporter(Reporter):
    def __init__(self, generator, presenter, i18n):
        # type: (Generator, Presenter, I18n) -> None
        self.i18n = i18n
        super(I18nReporter, self).__init__(generator, I18nPresenter(presenter, self.i18n))


class I18nPresenter(Presenter):
    def __init__(self, base_presenter, i18n):
        # type: (Presenter, I18n) -> None
        self.base_presenter = base_presenter
        self.i18n = i18n

    def present(self, dto):
        report = self.base_presenter.present(dto)
        if isinstance(report, I18nReport):
            report.set_i18n(self.i18n)

        return report



import logging

from bustoken import TK_MAINTENANCE_TERMINATE_REQUEST, TK_MAINTENANCE_RESTART_REQUEST, \
    TK_MAINTENANCE_RECREATE_FILES_XML, TK_MAINTENANCE_RECREATE_FILE_NUMBER, TK_MAINTENANCE_CLEAN_AUDIT_LOG
from messagehandler import EventHandler
from msgbus import MBEasyContext, TK_SYS_ACK, TK_SYS_NAK
from systools import sys_log_exception

from events import MaintenanceProcessor, CleanDatabasesProcessor, CleanJsonProcessors

logger = logging.getLogger("Maintenance")


class MaintenanceEventHandler(EventHandler):
    def __init__(self, mbcontext, maintenance_processor,  clean_databases_processor, clean_json_processor):
        # type: (MBEasyContext, MaintenanceProcessor, CleanDatabasesProcessor, CleanJsonProcessors) -> None
        super(MaintenanceEventHandler, self).__init__(mbcontext)
        self.mbcontext = mbcontext
        self.maintenance_processor = maintenance_processor
        self.clean_databases_processor = clean_databases_processor
        self.clean_json_processor = clean_json_processor

    def subscribe_events(self, subject):
        return

    def terminate_event(self):
        return

    def get_handled_tokens(self):
        return [TK_MAINTENANCE_TERMINATE_REQUEST,
                TK_MAINTENANCE_RESTART_REQUEST,
                TK_MAINTENANCE_RECREATE_FILES_XML,
                TK_MAINTENANCE_RECREATE_FILE_NUMBER]

    def handle_event(self, subject, evt_type, data, msg):
        logger.info("Event received: [{}]".format(subject))
        if not self.maintenance_processor.is_inside_maintenance_window():
            logger.info("Not inside maintenance window. Exiting...")
            return

        if subject == "CleanDatabases":
            self.clean_databases_processor.clean_databases()
        elif subject == "VacuumDatabases":
            self.clean_databases_processor.vacuum_databases()
        elif subject == "CleanJson":
            self.clean_json_processor.clean_json()

    def handle_message(self, msg):
        try:
            if msg.token == TK_MAINTENANCE_TERMINATE_REQUEST:
                response = self.maintenance_processor.handle_terminate_request(msg)
            elif msg.token == TK_MAINTENANCE_RESTART_REQUEST:
                response = self.maintenance_processor.handle_restart_request(msg)
            elif msg.token == TK_MAINTENANCE_CLEAN_AUDIT_LOG:
                response = self.maintenance_processor.clean_audit_log(msg)
            else:
                response = (False, "")

            msg.token = TK_SYS_ACK if response[0] else TK_SYS_NAK
            self.mbcontext.MB_ReplyMessage(msg, data=response[1])
        except Exception as ex:
            sys_log_exception("Unexpected exception handling event: {}".format(msg.token))
            msg.token = TK_SYS_NAK
            self.mbcontext.MB_ReplyMessage(msg, data=str(ex))



# encoding: utf-8

import json
from typing import List, Dict

from benefitsreport.commons import discount_in_all_items, check_item_in_action
from benefitsreport.model import OrderBenefit, OrderBenefitInfo
from old_helper import BaseRepository


class BenefitsOrderRepository(BaseRepository):

    def __init__(self, mb_context, pos_list):
        super(BenefitsOrderRepository, self).__init__(mb_context)

        self.pos_list = pos_list

    def get_orders_benefit_detailed(self, initial_date, end_date, operator_id, benefits):
        # type: (str, str, str, Dict) -> List[OrderBenefit]

        def inner_func(conn):

            query_operator = " and SessionId like '%user={0},%'".format(operator_id) if operator_id is not None else ""
            query = self._OrdersQuery.format(initial_date, end_date, query_operator)
            orders = []
            benefit_keys = [x.get("key") for x in benefits]
            for row in conn.select(query):
                benefit_list_json = json.loads(row.get_entry("BenefitList"))
                benefit_list = json.loads(benefit_list_json.get("benefitList", None))
                for benefit in benefit_list:
                    benefit_key = benefit.get("benefit").get("name")
                    if benefit_key not in benefit_keys:
                        continue
                    benefit_id = benefit.get("benefit").get("id")
                    benefit_applied = benefit.get("benefit_applied", False)
                    benefit_applied_id = benefit.get("benefit_key", None)
                    benefit_json = benefit.get("benefit", {})
                    benefit_applied = benefit_applied and "{0}_".format(benefit_id) in benefit_applied_id

                    if benefit_applied and self._check_item_to_add(benefit_json, row):
                        discount_items_values = benefit_list_json.get("discountItemValues", {})
                        orders.append(OrderBenefit(row, discount_items_values, OrderBenefitInfo.from_json(benefit)))

            return orders

        return self.execute_parallel_in_all_databases_returning_flat_list(inner_func, self.pos_list)

    def get_orders_benefit_consolidated(self, initial_date, end_date, operator_id):
        # type: (str, str, str) -> List[OrderBenefit]

        def inner_func(conn):
            query_operator = " and SessionId like '%user={0},%'".format(operator_id) if operator_id is not None else ""
            query = self._OrdersQueryConsolidated.format(initial_date, end_date, query_operator)
            orders = []
            for row in conn.select(query):
                benefit_list_json = json.loads(row.get_entry("BenefitList"))
                benefit_list = json.loads(benefit_list_json.get("benefitList", None))
                for benefit in benefit_list:
                    benefit_json = benefit.get("benefit", {})
                    benefit_applied = benefit.get("benefit_applied", False)
                    if benefit_applied and self._check_item_to_add(benefit_json, row):
                        discount_items_values = benefit_list_json.get("discountItemValues", {})
                        orders.append(OrderBenefit(row, discount_items_values, OrderBenefitInfo.from_json(benefit)))

            return orders

        return self.execute_parallel_in_all_databases_returning_flat_list(inner_func, self.pos_list)

    @staticmethod
    def _check_item_to_add(benefit, item):
        part_code = item.get_entry("PartCode")
        return discount_in_all_items(benefit) or check_item_in_action(benefit, part_code)

    _OrdersQuery = """
                    SELECT o.OrderId,
                           o.BusinessPeriod,
                           it.PartCode,
                           it.ItemId,
                           it.Level,
                           P.ProductName,
                            COALESCE(CASE WHEN it.DefaultQty IS NULL
                            THEN it.OrderedQty 
                            ELSE
                                CASE WHEN it.OrderedQty IS NULL
                                THEN it.DefaultQty 
                                ELSE
                                    CASE WHEN it.DefaultQty > it.OrderedQty
                                    THEN 0
                                    ELSE it.OrderedQty - it.DefaultQty
                                    END
                                END
                            END, 0)
                           AS OrderedQty,
                           it.LineNumber,
                           COALESCE(it.OverwrittenUnitPrice, op.DefaultUnitPrice, 0.0) as DefaultUnitPrice,
                           ocp.Value AS BenefitList,
                           oic.Key AS discountType,
                           oic.Value AS discountKey
                     FROM Orders o
                     INNER JOIN OrderCustomProperties ocp ON ocp.OrderId = o.OrderId AND ocp.Key = 'BENEFIT_LIST'
                     INNER JOIN OrderItem it ON o.OrderId = it.OrderId
                     INNER JOIN productdb.Product P ON  it.PartCode = P.ProductCode
                     INNER JOIN OrderItemCustomProperties oic
                        ON o.OrderId = oic.OrderId
                       AND it.PartCode = oic.PartCode
                       AND it.LineNumber = oic.LineNumber
                       AND (oic.Key = 'benefitKey' OR oic.Key ='servicePeriod')
                      LEFT JOIN OrderItem comboItem 
                        ON comboItem.OrderId = it.OrderId 
                       AND comboItem.LineNumber = it.LineNumber 
                       AND comboItem.ItemId = it.ItemId ||'.'||it.PartCode
                     INNER JOIN OrderPrice op 
                        ON op.OrderId = it.OrderId AND op.PriceKey = IFNULL(it.PriceKey,comboItem.PriceKey)
                     WHERE 
                        o.BusinessPeriod >= '{0}' 
                        AND o.BusinessPeriod <= '{1}' 
                        AND o.StateId = 5
                        {2}
                     GROUP by o.OrderId, o.BusinessPeriod, it.PartCode, oic.Value
                     ORDER BY o.OrderId, o.BusinessPeriod, P.ProductName
                  """

    _OrdersQueryConsolidated = """
                        SELECT o.OrderId,
                               o.BusinessPeriod,
                               it.PartCode,
                               it.ItemId,
                               it.Level,
                                '' as ProductName,
                                   COALESCE(
                                        CASE WHEN it.DefaultQty IS NULL
                                        THEN it.OrderedQty 
                                        ELSE
                                            CASE WHEN it.OrderedQty IS NULL
                                                THEN it.DefaultQty 
                                            ELSE
                                                CASE WHEN it.DefaultQty > it.OrderedQty
                                                THEN 0
                                                ELSE it.OrderedQty - it.DefaultQty
                                                END
                                            END
                                        END
                                   , 0)
                                AS OrderedQty,
                               it.LineNumber,
                               COALESCE(it.OverwrittenUnitPrice, op.DefaultUnitPrice, 0.0) as DefaultUnitPrice,
                               ocp.Value as BenefitList,
                               oic.Key as discountType,
                               oic.Value as discountKey
                          FROM Orders o
                         INNER JOIN OrderCustomProperties ocp ON ocp.OrderId = o.OrderId AND ocp.Key = 'BENEFIT_LIST'
                         INNER JOIN OrderItem it ON o.OrderId = it.OrderId
                         INNER JOIN OrderItemCustomProperties oic ON o.OrderId = oic.OrderId
                           AND it.PartCode = oic.PartCode and it.LineNumber = oic.LineNumber
                           AND (oic.Key = 'benefitKey' or oic.Key ='servicePeriod')
                          LEFT JOIN OrderItem comboItem 
                            ON comboItem.OrderId = it.OrderId 
                           AND comboItem.LineNumber = it.LineNumber 
                           AND comboItem.ItemId = it.ItemId ||'.'||it.PartCode
                         INNER JOIN OrderPrice op 
                            ON op.OrderId = it.OrderId AND op.PriceKey = IFNULL(it.PriceKey,comboItem.PriceKey)
                         WHERE o.BusinessPeriod >= '{0}' and o.BusinessPeriod <= '{1}' AND o.StateId = 5
                            {2}
                         GROUP by o.OrderId, o.BusinessPeriod, it.PartCode, oic.Value
                         ORDER BY o.OrderId, o.BusinessPeriod
                      """



# encoding: utf-8
import json

from hourlysalesreport import HourlySalesReport
from hourlysalesreport.repository import OrderRepository as HourlySalesOrderRepository


class ReportGenerator(object):
    @staticmethod
    def generate(mb_context, pos_id, pos_list, operator_id, period):
        pos_list = json.loads(pos_list)
        hourly_sales_report_generator = HourlySalesReport(HourlySalesOrderRepository(mb_context, pos_list))
        return hourly_sales_report_generator.generate_hourly_sales_report(pos_id, operator_id, period)




import logging

from sysactions import get_model, get_posot, has_current_order, get_current_order

logger = logging.getLogger("PosActions")


def check_active_order(pos_id, model=None):
    if model is None:
        model = get_model(pos_id)
    pos_ot = get_posot(model)
    if has_current_order(model):
        try:
            pos_ot.blkopnotify = True
            order = get_current_order(model)
            order_id = order.get("orderId")
            logger.info("POS has opened Order - Store and Proceed (Order Id: {})".format(order_id))
            pos_ot.storeOrder(pos_id)
        finally:
            pos_ot.blkopnotify = False



from logging import Logger

from production.box import OrderPropertiesFilterBox
from production.treebuilder import BoxConfiguration
from typing import Dict

from ._BoxBuilder import BoxBuilder


class OrderPropertiesFilterBoxBuilder(BoxBuilder):
    def __init__(
            self,
            loggers         # type: Dict[str, Logger]
    ):
        # type: (...) -> None

        super(OrderPropertiesFilterBoxBuilder, self).__init__(loggers)

    def build(
            self,
            config  # type: BoxConfiguration
    ):
        # type: (...) -> OrderPropertiesFilterBoxBuilder

        return OrderPropertiesFilterBox(
            config.name,
            config.sons,
            self.get_extra(config, "OrderProperty", ""),
            self.get_extra(config, "AllowedValues", []),
            self.get_extra(config, "ExcludedValues", []),
            self.get_extra(config, "RemoveOrder", True),
            self.get_logger(config)
        )



from production.box import CustomParamPointerBox
from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder


class CustomParamPointerBoxBuilder(BoxBuilder):
    def __init__(self, product_repository, loggers):
        super(CustomParamPointerBoxBuilder, self).__init__(loggers)
        self.product_repository = product_repository

    def build(self, config):
        # type: (BoxConfiguration) -> CustomParamPointerBox
        return CustomParamPointerBox(config.name,
                                     config.sons,
                                     self.product_repository,
                                     int(self.get_extra(config, "DefaultPoints", 0)),
                                     self.get_extra(config, "CountMergedItems", "true").lower() == "true",
                                     self.get_logger(config))



# -*- coding: utf-8 -*-
import sysactions

from actions.void.do_void_remote_order import do_void_remote_order
from custom_sysactions import user_authorized_to_execute_action
from systools import sys_log_exception


@sysactions.action
@user_authorized_to_execute_action
def do_ask_void_remote_order(pos_id, remote_id):
    try:
        message = "$CONFIRM_VOID_ORDER"
        confirmation = sysactions.show_messagebox(pos_id, message,
                                                  title="$TABLE_INFO", timeout=720000, buttons="$OK|$CANCEL")
        if confirmation == 1:
            return False

        error_message = do_void_remote_order(pos_id, remote_id)
        if not error_message:
            sysactions.show_messagebox(pos_id, "$REMOTE_ORDER_VOIDED_SUCCESS")
        else:
            sysactions.show_messagebox(pos_id, error_message)

    except Exception as ex:
        sys_log_exception("Error voiding delivery order")
        sysactions.show_messagebox(pos_id, "$ERROR_VOIDING_REMOTE_ORDER|{}".format(ex))



# -*- coding: utf-8 -*-

import utils
import actions.util
from actions.login.get_user_info import get_user_info
from model.customexception import GettingUserInfoException, CanceledDialogException

from sysactions import (
    action,
    get_model,
    show_messagebox,
    StopAction,
    AuthenticationFailed
)

from login_user import sign_in_quick_user
from signin_user import sign_in_table_user
from actions.pos import pos_must_be_blocked
from typing import Optional


@action
def common_login(
        pos_id,         # type: str
        user_id=None,   # type: Optional[str]
        password=None   # type: Optional[str]
):
    # type: (...) -> bool

    model = get_model(pos_id)
    _login_validations(model, pos_id, user_id)

    try:
        user_info = get_user_info(pos_id, user_id, password, model)
    except CanceledDialogException as _:
        raise StopAction()
    except (AuthenticationFailed, GettingUserInfoException) as _:
        show_messagebox(pos_id, "$INVALID_USERNAME_OR_PASSWORD", "$INFORMATION")
        raise StopAction()

    working_mode = model.find("WorkingMode").get("usrCtrlType")
    if working_mode == "QSR":
        return sign_in_quick_user(pos_id, user_info)
    else:
        return sign_in_table_user(pos_id, user_info)


def _login_validations(model, pos_id, user_id):
    if pos_must_be_blocked(pos_id):
        show_messagebox(pos_id, message="$POS_BLOCKED", icon="error")
        raise StopAction()

    if not utils.check_current_day(pos_id):
        raise StopAction()

    if actions.util.is_user_already_logged(pos_id, user_id):
        raise StopAction()



from abc import ABCMeta, abstractmethod


class NotificationRepository(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def notify(self, name, data):
        # type: (str, str) -> None

        raise NotImplementedError()



from typing import List

from basereport import TableReport, ReportColumnDefinition, AlignTypes
from report import Part
from report.command import AlignCommand, RepeatCommand, FontCommand
from ._ReportBuilder import ReportBuilder
from ..dto import TableReportHeaderDto


class TableReportHeader(ReportBuilder):
    def __init__(self, table_report_header_dto, pre_account_with_discount):
        # type: (TableReportHeaderDto, bool) -> None
        self.table_report_header_dto = table_report_header_dto
        self.pre_account_with_discount = pre_account_with_discount

    def generate(self):
        # type: () -> List[Part]

        report_header = []
        if not self.pre_account_with_discount:
            report_header.extend(
                    [
                        Part("$REPORT_ITEMS_CONFERENCE", [AlignCommand(AlignCommand.Alignment.center)]),
                        a_new_line(),
                        a_new_line()
                    ]
            )

        return self._format_report_header(self.pre_account_with_discount, report_header)

    def _format_report_header(self, show_order_id, report_header):
        report_header.extend([
            Part(commands=[FontCommand(FontCommand.Font.A)]),
            Part("$TABLE {}".format(self.table_report_header_dto.table_name[:20]),
                 [AlignCommand(AlignCommand.Alignment.center)]),
            a_new_line()
        ])

        if show_order_id:
            order_id_part = Part(
                    "$SHORT_REFERENCE {}".format(self.table_report_header_dto.order_id[:20]),
                    [AlignCommand(AlignCommand.Alignment.center)]
            )
            report_header.extend([order_id_part, a_new_line()])

        report_header.extend([
            Part("$OPERATOR_LABEL {}".format(self.table_report_header_dto.user_name[:20]),
                 [AlignCommand(AlignCommand.Alignment.center)]),
            a_new_line(),
            Part("$SEATS_NUMBER {}".format(self.table_report_header_dto.service_seats),
                 [AlignCommand(AlignCommand.Alignment.center)]),
            a_new_line(),
            a_new_line(),
            TableReport(lines=[["$START_HOUR", self._start_hour()]], column_definitions=a_time_column_definition()),
            TableReport(
                    lines=[["$TOTALED_HOUR", self._totaled_hour()]],
                    column_definitions=a_time_column_definition()
            ),
            a_new_line(),
            Part(commands=[RepeatCommand(38, "=")]),
            a_new_line(),

        ])
        return report_header

    def _start_hour(self):
        return "#DATETIME({})".format(self.table_report_header_dto.start_date.isoformat())

    def _totaled_hour(self):
        return "#DATETIME({})".format(self.table_report_header_dto.totaled_date.isoformat())


def a_new_line(commands=None):
    return Part("\n", commands=commands)


def a_time_column_definition():
    return [
        ReportColumnDefinition(width=13,
                               fill_with_char=" ",
                               before_text="",
                               align=AlignTypes.LEFT),
        ReportColumnDefinition(width=25,
                               fill_with_char="",
                               before_text="",
                               align=AlignTypes.RIGHT)

    ]



#
# The Python Imaging Library.
# $Id$
#
# a class to read from a container file
#
# History:
# 1995-06-18 fl     Created
# 1995-09-07 fl     Added readline(), readlines()
#
# Copyright (c) 1997-2001 by Secret Labs AB
# Copyright (c) 1995 by Fredrik Lundh
#
# See the README file for information on usage and redistribution.
#

##
# A file object that provides read access to a part of an existing
# file (for example a TAR file).

import io


class ContainerIO(object):
    def __init__(self, file, offset, length):
        """
        Create file object.

        :param file: Existing file.
        :param offset: Start of region, in bytes.
        :param length: Size of region, in bytes.
        """
        self.fh = file
        self.pos = 0
        self.offset = offset
        self.length = length
        self.fh.seek(offset)

    ##
    # Always false.

    def isatty(self):
        return False

    def seek(self, offset, mode=io.SEEK_SET):
        """
        Move file pointer.

        :param offset: Offset in bytes.
        :param mode: Starting position. Use 0 for beginning of region, 1
           for current offset, and 2 for end of region.  You cannot move
           the pointer outside the defined region.
        """
        if mode == 1:
            self.pos = self.pos + offset
        elif mode == 2:
            self.pos = self.length + offset
        else:
            self.pos = offset
        # clamp
        self.pos = max(0, min(self.pos, self.length))
        self.fh.seek(self.offset + self.pos)

    def tell(self):
        """
        Get current file pointer.

        :returns: Offset from start of region, in bytes.
        """
        return self.pos

    def read(self, n=0):
        """
        Read data.

        :param n: Number of bytes to read. If omitted or zero,
            read until end of region.
        :returns: An 8-bit string.
        """
        if n:
            n = min(n, self.length - self.pos)
        else:
            n = self.length - self.pos
        if not n:  # EOF
            return ""
        self.pos = self.pos + n
        return self.fh.read(n)

    def readline(self):
        """
        Read a line of text.

        :returns: An 8-bit string.
        """
        s = ""
        while True:
            c = self.read(1)
            if not c:
                break
            s = s + c
            if c == "\n":
                break
        return s

    def readlines(self):
        """
        Read multiple lines of text.

        :returns: A list of 8-bit strings.
        """
        lines = []
        while True:
            s = self.readline()
            if not s:
                break
            lines.append(s)
        return lines



import logging
import logging.config
import logging.handlers
import os
import sys
import json

import persistence
import unicodedata
from enum import Enum
from typing import Union


def config_logger(loader_path,
                  log_name='component',
                  configure_root=False,
                  root_level=logging.WARN,
                  max_files=5,
                  max_size=5242880,
                  add_error_handler=False):
    real_path = os.path.realpath(loader_path)
    if os.path.isfile(real_path) and real_path.endswith(".ini"):
        logging.config.fileConfig(real_path, disable_existing_loggers=False)
        return

    if os.path.isfile(real_path) and real_path.endswith(".json"):
        with open(real_path, "rb") as f:
            logging.config.dictConfig(json.loads(f.read().decode("utf-8")))
        return

    if os.path.isdir(real_path):
        dir_path = real_path
    else:
        dir_path = os.path.dirname(real_path)

    filename = os.path.join(dir_path, log_name + '.log')
    filename_error = os.path.join(dir_path, log_name + '.error.log')

    log_string = "%(asctime)-6s: %(name)s - %(levelname)s - %(thread)d - %(threadName)s - %(message)s"
    formatter = logging.Formatter(log_string)

    comp_logger = logging.getLogger(log_name)
    comp_logger.setLevel(logging.DEBUG)
    comp_logger.propagate = False

    rotating_file_logger = logging.handlers.RotatingFileHandler(filename, 'a', max_size, max_files)
    rotating_file_logger.setLevel(logging.DEBUG)
    rotating_file_logger.setFormatter(formatter)
    comp_logger.addHandler(rotating_file_logger)

    if add_error_handler:
        rotating_file_logger_error = logging.handlers.RotatingFileHandler(filename_error, 'a', max_size, max_files)
        rotating_file_logger_error.setLevel(logging.ERROR)
        rotating_file_logger_error.setFormatter(formatter)
        comp_logger.addHandler(rotating_file_logger_error)

    if configure_root:
        root_logger = logging.getLogger()
        root_logger.addHandler(rotating_file_logger)
        root_logger.setLevel(root_level)


def import_pydevd(loader_path, port, suspend=False):
    # type: (str, int, bool) -> None
    if "DEBUG" in os.environ:
        dir_path = os.path.dirname(os.path.realpath(loader_path))
        filename = os.path.join(dir_path, "debug.txt")
        if os.path.isfile(filename):
            with open(filename) as debug_file:
                debug_path = debug_file.read()
            if os.path.exists(debug_path):
                if debug_path not in sys.path:
                    sys.path.append(debug_path)

                import pydevd
                pydevd.settrace('localhost', port=port, stdoutToServer=True, stderrToServer=True, suspend=suspend)
            else:
                debug_path = os.path.abspath(debug_path)
                print(debug_path)
                print("path errado")


def get_valid_pricelists(mbcontext):
    price_lists = []

    conn = None
    try:
        conn = persistence.Driver().open(mbcontext)
        cursor = conn.select("SELECT PriceListID FROM Pricelist WHERE CURRENT_DATE BETWEEN EnabledFrom AND EnabledThru")
        for row in cursor:
            pl_id = str(row.get_entry(0) or '')
            price_lists.append(pl_id)
    finally:
        if conn:
            conn.close()
    return price_lists


def unicode_2_ascii(data):
    import unicodedata

    try:
        data = data.decode('latin_1', 'ignore')
    except Exception: # noqa
        pass
    finally:
        data = unicode(data)
        data = unicodedata.normalize('NFKD', data)
        data = data.encode('ascii', 'ignore')

    return data


def remove_accents(text):
    # type: (Union[str, unicode]) -> str

    if not text:
        return text

    try:
        text = unicode(text, 'utf-8')
    except (TypeError, NameError):
        pass

    text = unicodedata.normalize('NFD', text)
    text = text.encode('ascii', 'ignore')
    return text


class ExtendedEnum(Enum):

    @classmethod
    def list_values(cls):
        # type: () -> list
        return list(map(lambda enum: enum.value, cls))

    @classmethod
    def list_names(cls):
        # type: () -> list
        return list(map(lambda enum: enum.name, cls))

    @classmethod
    def list_enums_tuple(cls):
        # type: () -> list
        return list(map(lambda enum: (enum.name, enum.value), cls))

    @classmethod
    def get_name(cls, value):
        # type: (int) -> Union[str, None]
        if value not in cls.list_values():
            return None

        return next(str(enum.name) for enum in cls if enum.value == value)

    @classmethod
    def get_value(cls, name):
        # type: (str) -> Union[int, None]
        if name not in cls.list_names():
            return None

        return next(int(enum.value) for enum in cls if enum.name == name)


class MwOrderStatus(ExtendedEnum):
    IN_PROGRESS = 1
    STORED = 2
    TOTALED = 3
    VOIDED = 4
    PAID = 5
    RECALLED = 6



import copy
import os
import re

from .core import Argument
from .core import MultiCommand
from .core import Option
from .parser import split_arg_string
from .types import Choice
from .utils import echo

try:
    from collections import abc
except ImportError:
    import collections as abc

WORDBREAK = "="

# Note, only BASH version 4.4 and later have the nosort option.
COMPLETION_SCRIPT_BASH = """
%(complete_func)s() {
    local IFS=$'\n'
    COMPREPLY=( $( env COMP_WORDS="${COMP_WORDS[*]}" \\
                   COMP_CWORD=$COMP_CWORD \\
                   %(autocomplete_var)s=complete $1 ) )
    return 0
}

%(complete_func)setup() {
    local COMPLETION_OPTIONS=""
    local BASH_VERSION_ARR=(${BASH_VERSION//./ })
    # Only BASH version 4.4 and later have the nosort option.
    if [ ${BASH_VERSION_ARR[0]} -gt 4 ] || ([ ${BASH_VERSION_ARR[0]} -eq 4 ] \
&& [ ${BASH_VERSION_ARR[1]} -ge 4 ]); then
        COMPLETION_OPTIONS="-o nosort"
    fi

    complete $COMPLETION_OPTIONS -F %(complete_func)s %(script_names)s
}

%(complete_func)setup
"""

COMPLETION_SCRIPT_ZSH = """
#compdef %(script_names)s

%(complete_func)s() {
    local -a completions
    local -a completions_with_descriptions
    local -a response
    (( ! $+commands[%(script_names)s] )) && return 1

    response=("${(@f)$( env COMP_WORDS=\"${words[*]}\" \\
                        COMP_CWORD=$((CURRENT-1)) \\
                        %(autocomplete_var)s=\"complete_zsh\" \\
                        %(script_names)s )}")

    for key descr in ${(kv)response}; do
      if [[ "$descr" == "_" ]]; then
          completions+=("$key")
      else
          completions_with_descriptions+=("$key":"$descr")
      fi
    done

    if [ -n "$completions_with_descriptions" ]; then
        _describe -V unsorted completions_with_descriptions -U
    fi

    if [ -n "$completions" ]; then
        compadd -U -V unsorted -a completions
    fi
    compstate[insert]="automenu"
}

compdef %(complete_func)s %(script_names)s
"""

COMPLETION_SCRIPT_FISH = (
    "complete --no-files --command %(script_names)s --arguments"
    ' "(env %(autocomplete_var)s=complete_fish'
    " COMP_WORDS=(commandline -cp) COMP_CWORD=(commandline -t)"
    ' %(script_names)s)"'
)

_completion_scripts = {
    "bash": COMPLETION_SCRIPT_BASH,
    "zsh": COMPLETION_SCRIPT_ZSH,
    "fish": COMPLETION_SCRIPT_FISH,
}

_invalid_ident_char_re = re.compile(r"[^a-zA-Z0-9_]")


def get_completion_script(prog_name, complete_var, shell):
    cf_name = _invalid_ident_char_re.sub("", prog_name.replace("-", "_"))
    script = _completion_scripts.get(shell, COMPLETION_SCRIPT_BASH)
    return (
        script
        % {
            "complete_func": "_{}_completion".format(cf_name),
            "script_names": prog_name,
            "autocomplete_var": complete_var,
        }
    ).strip() + ";"


def resolve_ctx(cli, prog_name, args):
    """Parse into a hierarchy of contexts. Contexts are connected
    through the parent variable.

    :param cli: command definition
    :param prog_name: the program that is running
    :param args: full list of args
    :return: the final context/command parsed
    """
    ctx = cli.make_context(prog_name, args, resilient_parsing=True)
    args = ctx.protected_args + ctx.args
    while args:
        if isinstance(ctx.command, MultiCommand):
            if not ctx.command.chain:
                cmd_name, cmd, args = ctx.command.resolve_command(ctx, args)
                if cmd is None:
                    return ctx
                ctx = cmd.make_context(
                    cmd_name, args, parent=ctx, resilient_parsing=True
                )
                args = ctx.protected_args + ctx.args
            else:
                # Walk chained subcommand contexts saving the last one.
                while args:
                    cmd_name, cmd, args = ctx.command.resolve_command(ctx, args)
                    if cmd is None:
                        return ctx
                    sub_ctx = cmd.make_context(
                        cmd_name,
                        args,
                        parent=ctx,
                        allow_extra_args=True,
                        allow_interspersed_args=False,
                        resilient_parsing=True,
                    )
                    args = sub_ctx.args
                ctx = sub_ctx
                args = sub_ctx.protected_args + sub_ctx.args
        else:
            break
    return ctx


def start_of_option(param_str):
    """
    :param param_str: param_str to check
    :return: whether or not this is the start of an option declaration
        (i.e. starts "-" or "--")
    """
    return param_str and param_str[:1] == "-"


def is_incomplete_option(all_args, cmd_param):
    """
    :param all_args: the full original list of args supplied
    :param cmd_param: the current command paramter
    :return: whether or not the last option declaration (i.e. starts
        "-" or "--") is incomplete and corresponds to this cmd_param. In
        other words whether this cmd_param option can still accept
        values
    """
    if not isinstance(cmd_param, Option):
        return False
    if cmd_param.is_flag:
        return False
    last_option = None
    for index, arg_str in enumerate(
        reversed([arg for arg in all_args if arg != WORDBREAK])
    ):
        if index + 1 > cmd_param.nargs:
            break
        if start_of_option(arg_str):
            last_option = arg_str

    return True if last_option and last_option in cmd_param.opts else False


def is_incomplete_argument(current_params, cmd_param):
    """
    :param current_params: the current params and values for this
        argument as already entered
    :param cmd_param: the current command parameter
    :return: whether or not the last argument is incomplete and
        corresponds to this cmd_param. In other words whether or not the
        this cmd_param argument can still accept values
    """
    if not isinstance(cmd_param, Argument):
        return False
    current_param_values = current_params[cmd_param.name]
    if current_param_values is None:
        return True
    if cmd_param.nargs == -1:
        return True
    if (
        isinstance(current_param_values, abc.Iterable)
        and cmd_param.nargs > 1
        and len(current_param_values) < cmd_param.nargs
    ):
        return True
    return False


def get_user_autocompletions(ctx, args, incomplete, cmd_param):
    """
    :param ctx: context associated with the parsed command
    :param args: full list of args
    :param incomplete: the incomplete text to autocomplete
    :param cmd_param: command definition
    :return: all the possible user-specified completions for the param
    """
    results = []
    if isinstance(cmd_param.type, Choice):
        # Choices don't support descriptions.
        results = [
            (c, None) for c in cmd_param.type.choices if str(c).startswith(incomplete)
        ]
    elif cmd_param.autocompletion is not None:
        dynamic_completions = cmd_param.autocompletion(
            ctx=ctx, args=args, incomplete=incomplete
        )
        results = [
            c if isinstance(c, tuple) else (c, None) for c in dynamic_completions
        ]
    return results


def get_visible_commands_starting_with(ctx, starts_with):
    """
    :param ctx: context associated with the parsed command
    :starts_with: string that visible commands must start with.
    :return: all visible (not hidden) commands that start with starts_with.
    """
    for c in ctx.command.list_commands(ctx):
        if c.startswith(starts_with):
            command = ctx.command.get_command(ctx, c)
            if not command.hidden:
                yield command


def add_subcommand_completions(ctx, incomplete, completions_out):
    # Add subcommand completions.
    if isinstance(ctx.command, MultiCommand):
        completions_out.extend(
            [
                (c.name, c.get_short_help_str())
                for c in get_visible_commands_starting_with(ctx, incomplete)
            ]
        )

    # Walk up the context list and add any other completion
    # possibilities from chained commands
    while ctx.parent is not None:
        ctx = ctx.parent
        if isinstance(ctx.command, MultiCommand) and ctx.command.chain:
            remaining_commands = [
                c
                for c in get_visible_commands_starting_with(ctx, incomplete)
                if c.name not in ctx.protected_args
            ]
            completions_out.extend(
                [(c.name, c.get_short_help_str()) for c in remaining_commands]
            )


def get_choices(cli, prog_name, args, incomplete):
    """
    :param cli: command definition
    :param prog_name: the program that is running
    :param args: full list of args
    :param incomplete: the incomplete text to autocomplete
    :return: all the possible completions for the incomplete
    """
    all_args = copy.deepcopy(args)

    ctx = resolve_ctx(cli, prog_name, args)
    if ctx is None:
        return []

    has_double_dash = "--" in all_args

    # In newer versions of bash long opts with '='s are partitioned, but
    # it's easier to parse without the '='
    if start_of_option(incomplete) and WORDBREAK in incomplete:
        partition_incomplete = incomplete.partition(WORDBREAK)
        all_args.append(partition_incomplete[0])
        incomplete = partition_incomplete[2]
    elif incomplete == WORDBREAK:
        incomplete = ""

    completions = []
    if not has_double_dash and start_of_option(incomplete):
        # completions for partial options
        for param in ctx.command.params:
            if isinstance(param, Option) and not param.hidden:
                param_opts = [
                    param_opt
                    for param_opt in param.opts + param.secondary_opts
                    if param_opt not in all_args or param.multiple
                ]
                completions.extend(
                    [(o, param.help) for o in param_opts if o.startswith(incomplete)]
                )
        return completions
    # completion for option values from user supplied values
    for param in ctx.command.params:
        if is_incomplete_option(all_args, param):
            return get_user_autocompletions(ctx, all_args, incomplete, param)
    # completion for argument values from user supplied values
    for param in ctx.command.params:
        if is_incomplete_argument(ctx.params, param):
            return get_user_autocompletions(ctx, all_args, incomplete, param)

    add_subcommand_completions(ctx, incomplete, completions)
    # Sort before returning so that proper ordering can be enforced in custom types.
    return sorted(completions)


def do_complete(cli, prog_name, include_descriptions):
    cwords = split_arg_string(os.environ["COMP_WORDS"])
    cword = int(os.environ["COMP_CWORD"])
    args = cwords[1:cword]
    try:
        incomplete = cwords[cword]
    except IndexError:
        incomplete = ""

    for item in get_choices(cli, prog_name, args, incomplete):
        echo(item[0])
        if include_descriptions:
            # ZSH has trouble dealing with empty array parameters when
            # returned from commands, use '_' to indicate no description
            # is present.
            echo(item[1] if item[1] else "_")

    return True


def do_complete_fish(cli, prog_name):
    cwords = split_arg_string(os.environ["COMP_WORDS"])
    incomplete = os.environ["COMP_CWORD"]
    args = cwords[1:]

    for item in get_choices(cli, prog_name, args, incomplete):
        if item[1]:
            echo("{arg}\t{desc}".format(arg=item[0], desc=item[1]))
        else:
            echo(item[0])

    return True


def bashcomplete(cli, prog_name, complete_var, complete_instr):
    if "_" in complete_instr:
        command, shell = complete_instr.split("_", 1)
    else:
        command = complete_instr
        shell = "bash"

    if command == "source":
        echo(get_completion_script(prog_name, complete_var, shell))
        return True
    elif command == "complete":
        if shell == "fish":
            return do_complete_fish(cli, prog_name)
        elif shell in {"bash", "zsh"}:
            return do_complete(cli, prog_name, shell == "zsh")

    return False



from production.box import FilterDeletedItemsBeforeFirstStateBox
from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder


class FilterDeletedItemsBeforeFirstStateBoxBuilder(BoxBuilder):
    def __init__(self, loggers):
        super(FilterDeletedItemsBeforeFirstStateBoxBuilder, self).__init__(loggers)

    def build(self, config):
        # type: (BoxConfiguration) -> FilterDeletedItemsBeforeFirstStateBox
        return FilterDeletedItemsBeforeFirstStateBox(config.name,
                                                     config.sons,
                                                     self.get_extra(config, "OrderState", ""),
                                                     self.get_logger(config))



# This file is dual licensed under the terms of the Apache License, Version
# 2.0, and the BSD License. See the LICENSE file in the root of this repository
# for complete details.

from __future__ import absolute_import, division, print_function

import abc
import ipaddress
import warnings
from email.utils import parseaddr

import six
from six.moves import urllib_parse

from cryptography import utils
from cryptography.x509.name import Name
from cryptography.x509.oid import ObjectIdentifier


_GENERAL_NAMES = {
    0: "otherName",
    1: "rfc822Name",
    2: "dNSName",
    3: "x400Address",
    4: "directoryName",
    5: "ediPartyName",
    6: "uniformResourceIdentifier",
    7: "iPAddress",
    8: "registeredID",
}


def _lazy_import_idna():
    # Import idna lazily becase it allocates a decent amount of memory, and
    # we're only using it in deprecated paths.
    try:
        import idna
        return idna
    except ImportError:
        raise ImportError(
            "idna is not installed, but a deprecated feature that requires it"
            " was used. See: https://cryptography.io/en/latest/faq/#importe"
            "rror-idna-is-not-installed"
        )


class UnsupportedGeneralNameType(Exception):
    def __init__(self, msg, type):
        super(UnsupportedGeneralNameType, self).__init__(msg)
        self.type = type


@six.add_metaclass(abc.ABCMeta)
class GeneralName(object):
    @abc.abstractproperty
    def value(self):
        """
        Return the value of the object
        """


@utils.register_interface(GeneralName)
class RFC822Name(object):
    def __init__(self, value):
        if isinstance(value, six.text_type):
            try:
                value.encode("ascii")
            except UnicodeEncodeError:
                value = self._idna_encode(value)
                warnings.warn(
                    "RFC822Name values should be passed as an A-label string. "
                    "This means unicode characters should be encoded via "
                    "idna. Support for passing unicode strings (aka U-label) "
                    "will be removed in a future version.",
                    utils.PersistentlyDeprecated2017,
                    stacklevel=2,
                )
        else:
            raise TypeError("value must be string")

        name, address = parseaddr(value)
        if name or not address:
            # parseaddr has found a name (e.g. Name <email>) or the entire
            # value is an empty string.
            raise ValueError("Invalid rfc822name value")

        self._value = value

    value = utils.read_only_property("_value")

    @classmethod
    def _init_without_validation(cls, value):
        instance = cls.__new__(cls)
        instance._value = value
        return instance

    def _idna_encode(self, value):
        idna = _lazy_import_idna()
        _, address = parseaddr(value)
        parts = address.split(u"@")
        return parts[0] + "@" + idna.encode(parts[1]).decode("ascii")

    def __repr__(self):
        return "<RFC822Name(value={0!r})>".format(self.value)

    def __eq__(self, other):
        if not isinstance(other, RFC822Name):
            return NotImplemented

        return self.value == other.value

    def __ne__(self, other):
        return not self == other

    def __hash__(self):
        return hash(self.value)


def _idna_encode(value):
    idna = _lazy_import_idna()
    # Retain prefixes '*.' for common/alt names and '.' for name constraints
    for prefix in ['*.', '.']:
        if value.startswith(prefix):
            value = value[len(prefix):]
            return prefix + idna.encode(value).decode("ascii")
    return idna.encode(value).decode("ascii")


@utils.register_interface(GeneralName)
class DNSName(object):
    def __init__(self, value):
        if isinstance(value, six.text_type):
            try:
                value.encode("ascii")
            except UnicodeEncodeError:
                value = _idna_encode(value)
                warnings.warn(
                    "DNSName values should be passed as an A-label string. "
                    "This means unicode characters should be encoded via "
                    "idna. Support for passing unicode strings (aka U-label) "
                    "will be removed in a future version.",
                    utils.PersistentlyDeprecated2017,
                    stacklevel=2,
                )
        else:
            raise TypeError("value must be string")

        self._value = value

    value = utils.read_only_property("_value")

    @classmethod
    def _init_without_validation(cls, value):
        instance = cls.__new__(cls)
        instance._value = value
        return instance

    def __repr__(self):
        return "<DNSName(value={0!r})>".format(self.value)

    def __eq__(self, other):
        if not isinstance(other, DNSName):
            return NotImplemented

        return self.value == other.value

    def __ne__(self, other):
        return not self == other

    def __hash__(self):
        return hash(self.value)


@utils.register_interface(GeneralName)
class UniformResourceIdentifier(object):
    def __init__(self, value):
        if isinstance(value, six.text_type):
            try:
                value.encode("ascii")
            except UnicodeEncodeError:
                value = self._idna_encode(value)
                warnings.warn(
                    "URI values should be passed as an A-label string. "
                    "This means unicode characters should be encoded via "
                    "idna. Support for passing unicode strings (aka U-label) "
                    " will be removed in a future version.",
                    utils.PersistentlyDeprecated2017,
                    stacklevel=2,
                )
        else:
            raise TypeError("value must be string")

        self._value = value

    value = utils.read_only_property("_value")

    @classmethod
    def _init_without_validation(cls, value):
        instance = cls.__new__(cls)
        instance._value = value
        return instance

    def _idna_encode(self, value):
        idna = _lazy_import_idna()
        parsed = urllib_parse.urlparse(value)
        if parsed.port:
            netloc = (
                idna.encode(parsed.hostname) +
                ":{}".format(parsed.port).encode("ascii")
            ).decode("ascii")
        else:
            netloc = idna.encode(parsed.hostname).decode("ascii")

        # Note that building a URL in this fashion means it should be
        # semantically indistinguishable from the original but is not
        # guaranteed to be exactly the same.
        return urllib_parse.urlunparse((
            parsed.scheme,
            netloc,
            parsed.path,
            parsed.params,
            parsed.query,
            parsed.fragment
        ))

    def __repr__(self):
        return "<UniformResourceIdentifier(value={0!r})>".format(self.value)

    def __eq__(self, other):
        if not isinstance(other, UniformResourceIdentifier):
            return NotImplemented

        return self.value == other.value

    def __ne__(self, other):
        return not self == other

    def __hash__(self):
        return hash(self.value)


@utils.register_interface(GeneralName)
class DirectoryName(object):
    def __init__(self, value):
        if not isinstance(value, Name):
            raise TypeError("value must be a Name")

        self._value = value

    value = utils.read_only_property("_value")

    def __repr__(self):
        return "<DirectoryName(value={})>".format(self.value)

    def __eq__(self, other):
        if not isinstance(other, DirectoryName):
            return NotImplemented

        return self.value == other.value

    def __ne__(self, other):
        return not self == other

    def __hash__(self):
        return hash(self.value)


@utils.register_interface(GeneralName)
class RegisteredID(object):
    def __init__(self, value):
        if not isinstance(value, ObjectIdentifier):
            raise TypeError("value must be an ObjectIdentifier")

        self._value = value

    value = utils.read_only_property("_value")

    def __repr__(self):
        return "<RegisteredID(value={})>".format(self.value)

    def __eq__(self, other):
        if not isinstance(other, RegisteredID):
            return NotImplemented

        return self.value == other.value

    def __ne__(self, other):
        return not self == other

    def __hash__(self):
        return hash(self.value)


@utils.register_interface(GeneralName)
class IPAddress(object):
    def __init__(self, value):
        if not isinstance(
            value,
            (
                ipaddress.IPv4Address,
                ipaddress.IPv6Address,
                ipaddress.IPv4Network,
                ipaddress.IPv6Network
            )
        ):
            raise TypeError(
                "value must be an instance of ipaddress.IPv4Address, "
                "ipaddress.IPv6Address, ipaddress.IPv4Network, or "
                "ipaddress.IPv6Network"
            )

        self._value = value

    value = utils.read_only_property("_value")

    def __repr__(self):
        return "<IPAddress(value={})>".format(self.value)

    def __eq__(self, other):
        if not isinstance(other, IPAddress):
            return NotImplemented

        return self.value == other.value

    def __ne__(self, other):
        return not self == other

    def __hash__(self):
        return hash(self.value)


@utils.register_interface(GeneralName)
class OtherName(object):
    def __init__(self, type_id, value):
        if not isinstance(type_id, ObjectIdentifier):
            raise TypeError("type_id must be an ObjectIdentifier")
        if not isinstance(value, bytes):
            raise TypeError("value must be a binary string")

        self._type_id = type_id
        self._value = value

    type_id = utils.read_only_property("_type_id")
    value = utils.read_only_property("_value")

    def __repr__(self):
        return "<OtherName(type_id={}, value={!r})>".format(
            self.type_id, self.value)

    def __eq__(self, other):
        if not isinstance(other, OtherName):
            return NotImplemented

        return self.type_id == other.type_id and self.value == other.value

    def __ne__(self, other):
        return not self == other

    def __hash__(self):
        return hash((self.type_id, self.value))



from . import StatisticBox


class OrdersCounterStatisticBox(StatisticBox):
    def __init__(self, name, sons, statistic_updater, logger, get_config):
        super(OrdersCounterStatisticBox, self).__init__(name, sons, statistic_updater, logger, get_config)
        self.orders = []

    def update_statistics(self, order):
        order_id = order.order_id
        if order_id not in self.orders:
            self.orders.append(order_id)

        return len(self.orders)



from abc import ABCMeta, abstractmethod

from nperipherals_core.model.device import Device


class WriterDevice(Device):
    __metaclass__ = ABCMeta

    @abstractmethod
    def write(self, data):
        # type: (unicode) -> None
        raise NotImplementedError()

    @abstractmethod
    def get_write_retries(self):
        # type: () -> int
        raise NotImplementedError()

    @abstractmethod
    def get_write_retry_sleep_in_seconds(self):
        # type: () -> float
        raise NotImplementedError()



from typing import List, Dict, Optional

from ._FoundRoute import FoundRoute
from ._HttpRouter import HttpRouter
from ._Route import Route


class SimpleHttpRouter(HttpRouter):
    def __init__(self, routes):
        # type: (List[Route]) -> None
        self.method_dict = MethodDict()
        for route in routes:
            self.method_dict.add_route(route)

    def get_route(self, route):
        return self.method_dict.get_route(route)


class InternalRoute(object):
    def __init__(self, route, url_regex):
        self.original_route = route
        self.url_regex = url_regex


class MethodDict(object):
    def __init__(self):
        self.method_routes = {}  # type: Dict[str, PathDict]

    def add_route(self, route):
        # type: (Route) -> None
        if route.method not in self.method_routes:
            self.method_routes[route.method] = PathDict()

        self.method_routes[route.method].add_route(route)

    def get_route(self, route):
        try:
            p = self.method_routes[route.method]
        except KeyError:
            return None
        return p.get_longest(route.path)


class PathDict(object):
    def __init__(self):
        self.fixed_routes = {}  # type: Dict[str, List[PathDict]]
        self.variable_routes = []  # type: List[PathDict]
        self.final_paths = {}  # type: Dict[str, RouteNode]
        self.variable_name = None  # type: Optional[str]
        self.variable_route = None  # type: Optional[RouteNode]

    def add_route(self, route):
        # type: (Route) -> None
        parts = route.path[1:].split("/")
        self.add_route_to_dicts(parts, route, 0)

    def get_longest(self, path):
        # type: (str) -> Optional[FoundRoute]
        path_parameters = {}
        available_routes = self.get_valid_nodes(path[1:].split("/"), path_parameters)
        if len(available_routes) == 0:
            return None
        longest_route = None
        for route in available_routes:
            if longest_route is None or longest_route < route:
                longest_route = route

        return FoundRoute(longest_route.route, longest_route.path_parameters)

    def add_route_to_dicts(self, path, route, height):
        # type: (List[str], Route, int) -> None
        if len(path) == 1:
            if path[0].startswith("{") and path[0].endswith("}"):
                self.variable_name = path[0][1:-1]
                self.variable_route = RouteNode(route, height)
            else:
                self.final_paths[path[0]] = RouteNode(route, height)
        else:
            if path[0].startswith("{") and path[0].endswith("}"):
                p = PathDict()
                p.add_route_to_dicts(path[1:], route, height + 1)
                self.variable_name = path[0][1:-1]
                self.variable_routes.append(p)
            else:
                if path[0] not in self.fixed_routes:
                    self.fixed_routes[path[0]] = []
                p = PathDict()
                p.add_route_to_dicts(path[1:], route, height + 1)
                self.fixed_routes[path[0]].append(p)

    def get_valid_nodes(self, path, variables):
        # type: (List[str], Dict[str, str]) -> List[FoundNode]
        available_nodes = []
        if len(self.final_paths) > 0 and len(path) == 1 and path[0] in self.final_paths:
            available_nodes.append(FoundNode(self.final_paths[path[0]].route,
                                             self.final_paths[path[0]].height,
                                             variables))
        elif self.variable_route is not None and self.variable_name is not None:
            variables[self.variable_name] = "/".join(path)
            available_nodes.append(FoundNode(self.variable_route.route,
                                             self.variable_route.height,
                                             variables))
            return available_nodes

        if len(path) == 1:
            return available_nodes

        path_dicts = []  # type: List[PathDict]
        if path[0] in self.fixed_routes:
            path_dicts.extend(self.fixed_routes[path[0]])
        if self.variable_name is not None:
            variables[self.variable_name] = path[0]
            path_dicts.extend(self.variable_routes)

        if path_dicts is not None:
            for path_dict in path_dicts:
                node_variables = variables.copy()
                available_nodes.extend(path_dict.get_valid_nodes(path[1:], node_variables))
        return available_nodes


class RouteNode(object):
    def __init__(self, route, height):
        # type: (Route, int) -> None
        self.route = route
        self.height = height


class FoundNode(object):
    def __init__(self, route, height, path_parameters):
        # type: (Route, int , Dict[str, str]) -> None
        self.route = route
        self.height = height
        self.path_parameters = path_parameters

    def __lt__(self, o):
        # type: (object) -> bool
        if self.height < o.height:
            return True
        elif self.height == o.height and len(self.path_parameters) > len(o.path_parameters):
            return True

        return False

    def __gt__(self, o):
        # type: (object) -> bool
        return not self.__lt__(o)




import json

import sysactions
from bustoken import TK_RUPTURA_GET_DISABLED, TK_RUPTURA_GET_ENABLED
from msgbus import TK_SYS_NAK, FM_STRING


def get_disabled_products():
    msg = sysactions.send_message("Ruptura", TK_RUPTURA_GET_DISABLED, FM_STRING, "UI")

    if msg.token == TK_SYS_NAK:
        raise Exception()

    return json.loads(msg.data)


def get_enabled_products():
    msg = sysactions.send_message("Ruptura", TK_RUPTURA_GET_ENABLED, FM_STRING, "UI")

    if msg.token == TK_SYS_NAK:
        raise Exception()

    return json.loads(msg.data)



# -*- coding: utf-8 -*-

import logging

from sysactions import mbcontext

logger = logging.getLogger("TableActions")


def publish_event(pos_id, action_type, event_type, event_data, event_source_pos_id=0):
    event_subject = "POS{}".format(pos_id)
    event_data = str(event_data)
    event_xml = """<Event subject="{0}" type="{1}">
                        <{2}>
                            {3}
                        </{2}>
                   </Event>""".format(event_subject, event_type, action_type, event_data)

    mbcontext.MB_EasyEvtSend(subject=event_subject,
                             type=event_type,
                             xml=event_xml,
                             synchronous=False,
                             sourceid=event_source_pos_id,
                             queue=None,
                             timeout=-1)



from sys import version_info

if version_info[0:2] < (2, 6):
    def bin(x):
        if x <= 1:
            return '0b'+str(x)
        else:
            return bin(x>>1) + str(x&1)
else:
    bin = bin



from old_helper import round_half_away_from_zero
from pos_model import SaleItem, TaxItem
from typing import List, Tuple, Dict, Optional
from msgbus import MBEasyContext, FM_PARAM, TK_SYS_ACK
from bustoken import TK_FISCALWRAPPER_GET_CST, TK_FISCALWRAPPER_GET_BASE_REDUCTION

from _FixedRateTaxCalculator import FixedRateTaxCalculator


class ReducedBaseAmountTaxCalculator(FixedRateTaxCalculator):
    def __init__(self, mbcontext, tax_code, tax_name, tax_index, rate, part_codes, params):
        # type: (MBEasyContext, unicode, unicode, unicode, float, Dict[int, int], unicode) -> None
        super(ReducedBaseAmountTaxCalculator, self).__init__(mbcontext, tax_code, tax_name, tax_index, rate, part_codes, params)
        self.mbcontext = mbcontext
        self.taxes_to_reduce = params.split('|') if params else []
        self.percentage_to_reduce = 0.0
        ret = mbcontext.MB_EasySendMessage("FiscalWrapper", token=TK_FISCALWRAPPER_GET_CST, format=FM_PARAM)
        if ret.token != TK_SYS_ACK:
            raise Exception("Waiting for FiscalWrapper to start")
        else:
            self.cst_dict = eval(ret.data)
        ret = mbcontext.MB_EasySendMessage("FiscalWrapper", token=TK_FISCALWRAPPER_GET_BASE_REDUCTION, format=FM_PARAM)
        if ret.token != TK_SYS_ACK:
            raise Exception("Waiting for FiscalWrapper to start")
        else:
            self.base_reduction_dict = eval(ret.data)

    def calculate_tax(self, sale_item, previous_taxes):
        # type: (SaleItem, List[Tuple[SaleItem, TaxItem]]) -> Optional[TaxItem]
        tax_item = super(ReducedBaseAmountTaxCalculator, self).calculate_tax(sale_item, previous_taxes)
        if tax_item is None:
            return None

        reduced_base_amount = 0.0
        for previous_tax in previous_taxes:
            if previous_tax[1].tax_name in self.taxes_to_reduce:
                reduced_base_amount += previous_tax[1].tax_amount_ad

        self.percentage_to_reduce = 0
        rate = self.rate

        cst_icms = int(self.cst_dict.get((str(sale_item.part_code), "CST_ICMS")))

        if self.tax_name == "ICMS":
            if cst_icms == 20:
                self.percentage_to_reduce = float(self.base_reduction_dict.get(str(sale_item.part_code), 0)) / 100
            elif cst_icms == 60:
                rate = 0

        elif self.tax_name in ("PIS", "COFINS") and cst_icms == 40:
            reduced_base_amount = 0

        if self.tax_name == "FCP":
            reduced_base_amount = 0
            if cst_icms != 0:
                self.percentage_to_reduce = float(self.base_reduction_dict.get(str(sale_item.part_code), 0)) / 100

        tax_item.base_amount_ad = (tax_item.base_amount_ad * (1 - self.percentage_to_reduce)) - reduced_base_amount
        if tax_item.base_amount_ad < 0:
            tax_item.base_amount_ad = 0

        tax_item.tax_amount_bd = round_half_away_from_zero(tax_item.base_amount_bd * rate, 6)
        tax_item.tax_amount_ad = round_half_away_from_zero(tax_item.base_amount_ad * rate, 6)

        return tax_item



import json
from logging import Logger

from typing import Dict, Optional, List
from messagebus import MessageBus, Event, Message, NakMessage
from messagehandler import MessageHandler

from ._MessageProcessor import MessageProcessor
from ._ApiRouter import ApiRouter
from ._ApiRequest import ApiRequest
from ._MessageProcessorExecutorFactory import MessageProcessorExecutorFactory


class MessageProcessorMessageHandler(MessageHandler):
    def __init__(self,
                 event_processors,
                 message_processors,
                 api_routers,
                 executor_factory,
                 subject_counter=None,
                 logger=None):
        # type: (Optional[Dict[str, MessageProcessor]], Optional[Dict[int, MessageProcessor]], Optional[Dict[str, ApiRouter]], MessageProcessorExecutorFactory, Dict[str, int], Logger) -> None  # noqa
        self.event_processors = event_processors
        self.message_processors = message_processors
        self.api_routers = api_routers
        self.executor_factory = executor_factory
        self.subject_counter = subject_counter
        self.logger = logger

    def handle_event(self, message_bus, event):
        # type: (MessageBus, Event) -> None
        try:

            if self.subject_counter is not None:
                self._count_event(event.subject)

            if self.event_processors is not None:
                if self._has_event_type(event) and self._event_type_is_in_event_processors(event):
                    self._create_executor_and_execute(message_bus, event, self._get_event_type_key(event))
                    return

                if self._event_subject_is_in_event_processors(event):
                    self._create_executor_and_execute(message_bus, event, self._get_subject_key(event))
                    return

            if self.api_routers is not None and event.subject in self.api_routers:
                api_request = self._create_api_request(event)
                api_router = self.api_routers[event.subject]
                message_processor = api_router.get_message_processor(api_request)
                if message_processor is None:
                    raise MessageProcessorMessageHandler.ApiRequestNotMapped(api_request)

                executor = self.executor_factory.build_executor(message_processor)
                executor.execute_processor(message_bus, None, event, api_request)
                return

            raise MessageProcessorMessageHandler.EventNotMapped(event.subject, event.event_type)
        except:
            if self.logger is not None:
                self.logger.exception("Exception while handling event")
            raise

    def _count_event(self, subject):
        # type: (str) -> None

        subject_count = self.subject_counter.get(subject, 0)
        self.subject_counter[subject] = subject_count + 1

    def _create_api_request(self, event):
        data = event.data
        if isinstance(data, ApiRequest):
            api_request = data
        elif isinstance(data, bytes):
            request = json.loads(data, encoding="utf-8")
            path_variables_names = self._get_path_variables_names(request["api_path"])
            path_variables = {}
            for path_variable in path_variables_names:
                if path_variable in request["query"]:
                    path_variables[path_variable] = request["query"][path_variable]
                    del request["query"][path_variable]

            query_variables = request["query"]

            api_request = ApiRequest(event.event_type,
                                     request["api_path"],
                                     path_variables,
                                     query_variables,
                                     request["headers"],
                                     request["body"])
        else:
            raise Exception("Invalid request type: " + str(type(data)))
        return api_request

    def handle_message(self, message_bus, message):
        # type: (MessageBus, Message) -> None
        try:
            if self.message_processors is not None and message.token in self.message_processors:
                executor = self.executor_factory.build_executor(self.message_processors[message.token])
                executor.execute_processor(message_bus, message, None, message.data)
            else:
                message_bus.reply_message(message, NakMessage())
        except:
            if self.logger is not None:
                self.logger.exception("Exception handling message")
            raise

    def _has_event_type(self, event):
        return event.event_type is not None

    def _event_type_is_in_event_processors(self, event):
        return self._get_event_type_key(event) in self.event_processors

    def _get_event_type_key(self, event):
        return event.subject + "_" + event.event_type

    def _event_subject_is_in_event_processors(self, event):
        return self._get_subject_key(event) in self.event_processors

    def _get_subject_key(self, event):
        return event.subject

    def _create_executor_and_execute(self, message_bus, event, event_key):
        message_processor = self.event_processors[event_key]
        executor = self.executor_factory.build_executor(message_processor)
        executor.execute_processor(message_bus, None, event, event.data)

    def _get_path_variables_names(self, api_path):
        # type: (str) -> List[str]
        ret = []

        index2 = 0
        while True:
            try:
                index1 = api_path.index("{", index2)
            except ValueError:
                return ret
            try:
                index2 = api_path.index("}", index1)
            except ValueError:
                return ret

            ret.append(api_path[index1 + 1:index2])

    class EventNotMapped(Exception):
        def __init__(self, subject, type):
            # type: (str, str) -> None
            self.subject = subject
            self.type = type

        def __str__(self):
            return "Event not mapped: {} - {}".format(self.subject, self.type)

        def __repr__(self):
            return "EventNotMapped('{}', '{}')".format(self.subject, self.type)

    class ApiRequestNotMapped(Exception):
        def __init__(self, api_request):
            # type: (ApiRequest) -> None
            self.api_request = api_request

        def __str__(self):
            return u"ApiRequest not mapped: {} - {}".format(self.api_request.method, self.api_request.api_path)

        def __repr__(self):
            return u"ApiRequest not mapped: {} - {}".format(self.api_request.method, self.api_request.api_path)



from datetime import datetime


class Session(object):
    def __init__(self, id, pos_id, operator_id, operator_name, business_period, open_time, close_time):
        # type: (str, int, int, str, datetime, datetime, datetime) -> None
        self.id = id
        self.pos_id = pos_id
        self.operator_id = operator_id
        self.operator_name = operator_name
        self.business_period = business_period
        self.open_time = open_time
        self.close_time = close_time



from _NfceXmlPartBuilder import NfceXmlPartBuilder
from pos_model import Order
from typing import Any, Dict


class TranspBuilder(NfceXmlPartBuilder):
    def build_xml(self, order, context):
        # type: (Order, Dict[unicode, Any]) -> unicode
        # Transporte
        inner_xml = "<transp><modFrete>9</modFrete></transp>"

        return inner_xml



from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder
from production.box import TagFilterBox


class TagFilterBoxBuilder(BoxBuilder):
    def __init__(self, loggers):
        super(TagFilterBoxBuilder, self).__init__(loggers)

    def build(self, config):
        # type: (BoxConfiguration) -> TagFilterBox
        return TagFilterBox(config.name,
                            config.sons,
                            self.get_extra(config, "AllowedTags", None),
                            self.get_extra(config, "ForbiddenTags", None),
                            self.get_extra(config, "AllowComboTags", 'False'),
                            self.get_logger(config))



from logging import Logger
from mw_helper import ensure_list
from mwrepository import ProductRepository
from production.model import ProductionOrder, Item
from production.view import ProductionView
from production.box._OrderChangerProductionBox import OrderChangerProductionBox
from typing import List


class ItemSorterBox(OrderChangerProductionBox):
    def __init__(self, name, sons, product_repository, sort_order, item_property_sorters, logger):
        # type: (str, ProductionView, ProductRepository, List[str], List[str], Logger) -> None

        super(ItemSorterBox, self).__init__(name, sons, logger)

        self.sort_order = None
        self.item_property_sorters = ensure_list(item_property_sorters)
        self.logger = logger

        if sort_order is not None:
            self.sort_order = {line: product_repository.get_part_codes_of_jit_lines([line]) for line in sort_order}

    def change_order(self, order):
        # type: (ProductionOrder) -> ProductionOrder

        order = self.sort_order_items(order)
        if self.item_property_sorters is None or len(order.items) == 0:
            return order

        self._sort_order_items_by_attributes(order)
        return order

    def _sort_order_items_by_attributes(self, order):
        # type: (ProductionOrder) -> ()

        try:
            order.items = sorted(order.items, key=lambda item: self._manager_item_filter(item))
        except Exception as _:
            self.logger.exception("Error item sorter box")

    def _manager_item_filter(self, item):
        # type: (Item) -> ()

        return [self._parse_value(getattr(item, property_name)) for property_name in self.item_property_sorters]

    @staticmethod
    def _parse_value(property_value):
        # type: (str) -> ()

        try:
            return int(property_value) if property_value.isdigit() else property_value
        except ValueError:
            return property_value

    def sort_order_items(self, order):
        # type: (ProductionOrder) -> ProductionOrder

        if self.sort_order is None:
            return order

        order_items = order.items
        order.items = []
        for jit_line in self.sort_order:
            items_to_remove = []
            for item in order_items:
                if item.level > 0:
                    continue

                if item.part_code in self.sort_order[jit_line]:
                    order.items.append(item)
                    items_to_remove.append(item)

            for item in items_to_remove:
                order_items.remove(item)

        for item in order_items:
            order.items.append(item)

        return order



from xml.etree import cElementTree as eTree

from application.model import DefaultBenefitApplierRepository, BenefitAppliers
from application.model.configuration import Configurations
from application.repository.pos import DBRepository
from application.repository.pos.utils import find_order_benefits_id
from typing import Dict


class OrderModifiedReopenedInteractor(object):
    
    def __init__(self, configs, db_repository, benefit_appliers_repositories):
        # type: (Configurations, DBRepository, Dict[BenefitAppliers: DefaultBenefitApplierRepository]) -> None
        
        self.configs = configs
        self.logger = self.configs.logger
        self.db_repository = db_repository
        self.benefit_appliers_repositories = benefit_appliers_repositories

    def remove_benefit(self, current_pos_id, originator_pos_id, order_id, order_picture):
        # type: (int, int, int, eTree) -> None

        order_benefits_id = find_order_benefits_id(self.logger, order_picture)
        if not order_benefits_id:
            return

        message = "Benefits founded to remove: {}; Current/Originator PosId: {}/{}; OrderId: {}"
        self.logger.info(message.format(order_benefits_id, current_pos_id, originator_pos_id, order_id))
        
        self.benefit_appliers_repositories[BenefitAppliers.N_DISCOUNT].remove_benefit(current_pos_id)



from xml.etree import cElementTree as eTree

from msgbus import TK_POS_LISTUSERS, FM_PARAM, TK_SYS_NAK
from sysactions import send_message


def get_pos_users(pos_id):
    msg = send_message("POS%d" % int(pos_id), TK_POS_LISTUSERS, FM_PARAM, "%s" % pos_id)
    if msg.token == TK_SYS_NAK:
        raise Exception("Fail listing operators")
    xml = eTree.XML(msg.data)
    return [tag for tag in xml.findall("User")]


def get_opened_pos_user(pos_id, user_id):
    pos_user = None
    user_list = get_pos_users(pos_id)
    user_filter = filter(lambda x: x.get("id") == str(user_id) and not x.get("closeTime"), user_list)
    if user_filter not in [None, [], ""]:
        pos_user = user_filter[0]
    return pos_user



from typing import Dict, Any
from pos_model import Order

from old_helper import validar_cnpj, validar_cpf

from _NfceXmlPartBuilder import NfceXmlPartBuilder
from _ContextKeys import ContextKeys


class DestinatarioBuilder(NfceXmlPartBuilder):

    def __init__(self, ambiente):
        # type: (unicode) -> None
        self.ambiente = ambiente

    def build_xml(self, order, context):
        # type: (Order, Dict[unicode, Any]) -> unicode

        # Dados Destinatario (cliente)
        xml = ""
        cpf_cnpj = order.customer_cpf
        context[ContextKeys.cpf_cnpj] = cpf_cnpj

        if cpf_cnpj:
            if len(cpf_cnpj) == 11 and validar_cpf(cpf_cnpj):
                xml += "<dest><CPF>{}</CPF>".format(cpf_cnpj)
            elif len(cpf_cnpj) == 14 and validar_cnpj(cpf_cnpj):
                xml += "<dest><CNPJ>{}</CNPJ>".format(cpf_cnpj)

            if not xml:
                return ""

            if self.ambiente == "2":
                xml += "<xNome>NF-E EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL</xNome>"

            xml += "<indIEDest>9</indIEDest>"
            xml += "</dest>"

        return xml



from production.box import ProdStateTimerBox
from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder


class ProdStateTimerBoxBuilder(BoxBuilder):
    def __init__(self, loggers):
        super(ProdStateTimerBoxBuilder, self).__init__(loggers)

    def build(self, config):
        # type: (BoxConfiguration) -> ProdStateTimerBox
        return ProdStateTimerBox(config.name,
                                 config.sons,
                                 self.get_extra(config, "ProdState", None),
                                 self.get_logger(config))



from ._TableReportHeader import TableReportHeader
from ._TableReportBody import TableReportBody
from ._ReportBuilder import ReportBuilder

__all__ = ["TableReportHeader", "TableReportBody", "ReportBuilder"]
